!MNH_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!!    ############################ 
      MODULE MODD_FOREFIRE_n
!!    ############################ 
!!    AUTHOR
!!    ------
!!	P. Tulet     *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    07/08
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_PARAMETERS, ONLY: JPMODELMAX

IMPLICIT NONE 

TYPE FOREFIRE_t

	! Effective coupling for this model
	LOGICAL		:: FFCOUPLING
	
	! Current time of the simulation
	REAL 			:: FF_TIME
	
	! Dimensions of the matrices of ForeFire
	INTEGER 		:: FF_MATRIXSIZE, FF_SVMATRIXSIZE
	INTEGER 		:: FF_NX, FF_NY, FF_NZ
	
	! Dimensions of the matrices for parallelization
	INTEGER 		:: FF_PARALMATRIXSIZE


	! INFORMATION TO BE PASSED FROM FOREFIRE TO MNH
	!----------------------------------------------
	
	! Boolean for dumping 3D outputs through ForeFire
	INTEGER		:: FFNMODEL
	INTEGER		:: PROCID
	INTEGER 		:: FF3DOUT
	LOGICAL 		:: FF3DOUTPUTSFLOW, FF3DOUTPUTSPHYS, FF3DOUTPUTSCHEM
	REAL 			:: FFOUTPUTSUPDATE
	REAL 			:: FFNUMOUT
	REAL 			:: FFREFERENCETIME
	
	! Maximum number of fire markers in each MNH cell
	INTEGER 		:: NBFNMAX
	
	! Coefficient for the exchange of information for the outer wind
	INTEGER 		:: FFMULT

	! Position and velocity of the fire markers in each MNH cell
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_POSX
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_POSY
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_VELX
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_VELY
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_TIME
	REAL, DIMENSION(:,:,:), POINTER :: FFNODES_ID
	
	! Heat flux stemming from ForeFire in each cell
	REAL, DIMENSION(:,:), POINTER 	:: FF_HEATFLUX
	! Vapor flux stemming from ForeFire in each cell
	REAL, DIMENSION(:,:), POINTER 	:: FF_VAPORFLUX
	! Scalar variables' fluxes stemming from ForeFire in each cell
	REAL, DIMENSION(:,:,:), POINTER :: FF_SVFLUXES
	! Chemical index of the variables in case of chemical coupling
	INTEGER, DIMENSION(:), POINTER :: FF_CHEMINDICES
	! Chemical variables' fluxes stemming from ForeFire in each cell
	REAL, DIMENSION(:,:,:), POINTER :: FF_CVFLUXES
	! Chemical index of the variables for outputs purpose
	INTEGER, DIMENSION(:), POINTER :: FF_CHEMINDOUT
	
	! Velocities in the outer region of the domain for continuity in the halo
	REAL, DIMENSION(:,:), POINTER 	:: FFOUTERWINDU
	REAL, DIMENSION(:,:), POINTER 	:: FFOUTERWINDV

END TYPE FOREFIRE_t

! Definition of the pointers for all the models
TYPE(FOREFIRE_t), DIMENSION(JPMODELMAX), TARGET, SAVE :: FOREFIRE_MODEL

! Definition of the current pointers
! Scalars
LOGICAL, POINTER 	:: FFCOUPLING=>NULL()
REAL, POINTER 		:: FF_TIME=>NULL()
INTEGER, POINTER 	:: FF_MATRIXSIZE=>NULL() 
INTEGER, POINTER 	:: FF_SVMATRIXSIZE=>NULL()
INTEGER, POINTER 	:: FF_NX=>NULL()
INTEGER, POINTER 	:: FF_NY=>NULL()
INTEGER, POINTER 	:: FF_NZ=>NULL()
INTEGER, POINTER 	:: FF_PARALMATRIXSIZE=>NULL()
INTEGER, POINTER 	:: FFNMODEL=>NULL()
INTEGER, POINTER 	:: PROCID=>NULL()
INTEGER, POINTER 	:: FF3DOUT=>NULL()
LOGICAL, POINTER 	:: FF3DOUTPUTSFLOW=>NULL()
LOGICAL, POINTER 	:: FF3DOUTPUTSPHYS=>NULL()
LOGICAL, POINTER 	:: FF3DOUTPUTSCHEM=>NULL()
REAL, POINTER 		:: FFOUTPUTSUPDATE=>NULL()
REAL, POINTER 		:: FFNUMOUT=>NULL()
REAL, POINTER 		:: FFREFERENCETIME=>NULL()
INTEGER, POINTER 	:: NBFNMAX=>NULL()
INTEGER, POINTER 	:: FFMULT=>NULL()
! Arrays
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_POSX=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_POSY=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_VELX=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_VELY=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_TIME=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FFNODES_ID=>NULL()
REAL, DIMENSION(:,:), POINTER 	:: FF_HEATFLUX=>NULL()
REAL, DIMENSION(:,:), POINTER 	:: FF_VAPORFLUX=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FF_SVFLUXES=>NULL()
REAL, DIMENSION(:,:,:), POINTER 	:: FF_CVFLUXES=>NULL()
INTEGER, DIMENSION(:), POINTER :: FF_CHEMINDICES=>NULL()
INTEGER, DIMENSION(:), POINTER :: FF_CHEMINDOUT=>NULL()
REAL, DIMENSION(:,:), POINTER 	:: FFOUTERWINDU=>NULL()
REAL, DIMENSION(:,:), POINTER 	:: FFOUTERWINDV=>NULL()

CONTAINS

SUBROUTINE FOREFIRE_GOTO_MODEL(KFROM, KTO)

INTEGER, INTENT(IN) :: KFROM, KTO

! Save current state for allocated arrays
FOREFIRE_MODEL(KFROM)%FFNODES_POSX=>FFNODES_POSX
FOREFIRE_MODEL(KFROM)%FFNODES_POSY=>FFNODES_POSY
FOREFIRE_MODEL(KFROM)%FFNODES_VELX=>FFNODES_VELX
FOREFIRE_MODEL(KFROM)%FFNODES_VELY=>FFNODES_VELY
FOREFIRE_MODEL(KFROM)%FFNODES_TIME=>FFNODES_TIME
FOREFIRE_MODEL(KFROM)%FFNODES_ID=>FFNODES_ID
FOREFIRE_MODEL(KFROM)%FF_HEATFLUX=>FF_HEATFLUX
FOREFIRE_MODEL(KFROM)%FF_VAPORFLUX=>FF_VAPORFLUX
FOREFIRE_MODEL(KFROM)%FF_SVFLUXES=>FF_SVFLUXES
FOREFIRE_MODEL(KFROM)%FF_CHEMINDICES=>FF_CHEMINDICES
FOREFIRE_MODEL(KFROM)%FF_CHEMINDOUT=>FF_CHEMINDOUT
FOREFIRE_MODEL(KFROM)%FF_CVFLUXES=>FF_CVFLUXES
FOREFIRE_MODEL(KFROM)%FFOUTERWINDU=>FFOUTERWINDU
FOREFIRE_MODEL(KFROM)%FFOUTERWINDV=>FFOUTERWINDV

! Current model is set to model KTO
! Scalars
FFCOUPLING=>FOREFIRE_MODEL(KTO)%FFCOUPLING
FF_TIME=>FOREFIRE_MODEL(KTO)%FF_TIME
FF_MATRIXSIZE=>FOREFIRE_MODEL(KTO)%FF_MATRIXSIZE
FF_SVMATRIXSIZE=>FOREFIRE_MODEL(KTO)%FF_SVMATRIXSIZE
FF_NX=>FOREFIRE_MODEL(KTO)%FF_NX
FF_NY=>FOREFIRE_MODEL(KTO)%FF_NY
FF_NZ=>FOREFIRE_MODEL(KTO)%FF_NZ
FF_PARALMATRIXSIZE=>FOREFIRE_MODEL(KTO)%FF_PARALMATRIXSIZE
FFNMODEL=>FOREFIRE_MODEL(KTO)%FFNMODEL
PROCID=>FOREFIRE_MODEL(KTO)%PROCID
FF3DOUT=>FOREFIRE_MODEL(KTO)%FF3DOUT
FF3DOUTPUTSFLOW=>FOREFIRE_MODEL(KTO)%FF3DOUTPUTSFLOW
FF3DOUTPUTSPHYS=>FOREFIRE_MODEL(KTO)%FF3DOUTPUTSPHYS
FF3DOUTPUTSCHEM=>FOREFIRE_MODEL(KTO)%FF3DOUTPUTSCHEM
FFOUTPUTSUPDATE=>FOREFIRE_MODEL(KTO)%FFOUTPUTSUPDATE
FFNUMOUT=>FOREFIRE_MODEL(KTO)%FFNUMOUT
FFREFERENCETIME=>FOREFIRE_MODEL(KTO)%FFREFERENCETIME
NBFNMAX=>FOREFIRE_MODEL(KTO)%NBFNMAX
FFMULT=>FOREFIRE_MODEL(KTO)%FFMULT
! Arrays
FFNODES_POSX=>FOREFIRE_MODEL(KTO)%FFNODES_POSX
FFNODES_POSY=>FOREFIRE_MODEL(KTO)%FFNODES_POSY
FFNODES_VELX=>FOREFIRE_MODEL(KTO)%FFNODES_VELX
FFNODES_VELY=>FOREFIRE_MODEL(KTO)%FFNODES_VELY
FFNODES_TIME=>FOREFIRE_MODEL(KTO)%FFNODES_TIME
FFNODES_ID=>FOREFIRE_MODEL(KTO)%FFNODES_ID
FF_HEATFLUX=>FOREFIRE_MODEL(KTO)%FF_HEATFLUX
FF_VAPORFLUX=>FOREFIRE_MODEL(KTO)%FF_VAPORFLUX
FF_SVFLUXES=>FOREFIRE_MODEL(KTO)%FF_SVFLUXES
FF_CHEMINDICES=>FOREFIRE_MODEL(KTO)%FF_CHEMINDICES
FF_CHEMINDOUT=>FOREFIRE_MODEL(KTO)%FF_CHEMINDOUT
FF_CVFLUXES=>FOREFIRE_MODEL(KTO)%FF_CVFLUXES
FFOUTERWINDU=>FOREFIRE_MODEL(KTO)%FFOUTERWINDU
FFOUTERWINDV=>FOREFIRE_MODEL(KTO)%FFOUTERWINDV

END SUBROUTINE FOREFIRE_GOTO_MODEL

END MODULE MODD_FOREFIRE_n
