!MNH_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!--------------- special set of characters for RCS information
!-----------------------------------------------------------------
! $Source$ $Revision$
! MASDEV4_7 rad 2006/08/29 10:53:39
!-----------------------------------------------------------------
!     ###########################
      MODULE MODI_INI_HOR_AERCLIM
!     ###########################
!
INTERFACE
!
      SUBROUTINE INI_HOR_AERCLIM( HAER, KIB, KIE, KJB, KJE, KDLON, KYMD, KHOURS,& 
                                  PLAT, PLON, PAESEA,PAELAND, PAEURB, PAEDES )
!
CHARACTER (LEN=*),      INTENT(IN) :: HAER       ! aerosol optical thickness climatology
INTEGER, INTENT(IN)               :: KIB ! I index value of the first inner mass point
INTEGER, INTENT(IN)               :: KIE ! I index value of the last inner mass point
INTEGER, INTENT(IN)               :: KJB ! J index value of the first inner mass point
INTEGER, INTENT(IN)               :: KJE ! J index value of the last inner mass point
INTEGER, INTENT(IN)               :: KDLON ! number of equivalent column in the mnh domain
INTEGER, INTENT(IN)               :: KYMD  ! date 
INTEGER, INTENT(IN)               :: KHOURS! hour
REAL, DIMENSION(:,:), INTENT(IN)  :: PLAT ! MNH lat array
REAL, DIMENSION(:,:), INTENT(IN)  :: PLON ! MNH lon array
REAL, DIMENSION(:),   INTENT(OUT) :: PAESEA  ! integrated optical thickness for sea aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAELAND ! integrated optical thickness for lan aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAEURB  ! integrated optical thickness for urb aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAEDES  ! integrated optical thickness for des aerosol 
!
END SUBROUTINE
!
END INTERFACE
!
END MODULE MODI_INI_HOR_AERCLIM
!
!     ###########################################################################
      SUBROUTINE INI_HOR_AERCLIM( HAER, KIB, KIE, KJB, KJE, KDLON, KYMD,KHOURS, &
                                  PLAT, PLON, PAESEA,PAELAND, PAEURB, PAEDES )
!     ###########################################################################
!
!
!     PURPOSE.
!     --------
!     ECMWF Radiation scheme initialisation.
!     Perform the initialisation of aerosol fields from CEP climatologies . Two climato are available
!      - The Tegen et al. (1997) mensual climatology,  load and time-interpolated in SUE~ routines.  
!      - The  Tanre (1984) T5 climatology for horizontal distribution of integrated optical 
!       thickness relative to 4 classes of aerosols ( sea, land, town, desert).The corresponding 
!       grid- mode file was provided by ARPEGE-CLIMAT team (thanks !)  
!     
!     Fitting to MesoNH domain by horizontal interpolation (with horribl routine).
!
!     METHOD.
!     -------
!     Loading of global regular grid and definition of global aerosols arrays (input).
!     Specification of the output grid characteristics from LAT and LON arrays for horribl interface.
!     Call to horribl 
!
!     AUTHORS.
!     --------
!     F. SOLMON  MAR 2002 
!__________________________________________________________________________________
!
!*       0.     DECLARATIONS
!               ------------
!
USE YOEAERC  , ONLY : RTAEBC, RTAEOR, RTAESD, RTAESS, RTAESU, RTAEVO
USE YOEAERD  , ONLY : RCAEOPS  ,RCAEOPL  ,RCAEOPU  ,RCAEOPD 
!
!
USE MODD_CONF
USE MODD_GRID
USE MODD_PARAMETERS
USE MODI_HORIBL
!
USE MODI_SUECAEBC
USE MODI_SUECAEOR
USE MODI_SUECAESD
USE MODI_SUECAESS
USE MODI_SUECAESU
USE MODI_SUECAEC
!
IMPLICIT NONE
!
!*       0.1   declarations of arguments
!
CHARACTER (LEN=*),      INTENT(IN) :: HAER ! aerosol optical thickness climatology
INTEGER, INTENT(IN)               :: KIB ! I index value of the first inner mass point
INTEGER, INTENT(IN)               :: KIE ! I index value of the last inner mass point
INTEGER, INTENT(IN)               :: KJB ! J index value of the first inner mass point
INTEGER, INTENT(IN)               :: KJE ! J index value of the last inner mass point
INTEGER, INTENT(IN)               :: KDLON ! number of equivalent column in the mnh domain
INTEGER, INTENT(IN)               :: KYMD   ! date ( time interpolation in climatology)
INTEGER, INTENT(IN)               :: KHOURS ! 
REAL, DIMENSION(:,:), INTENT(IN)  :: PLAT ! MNH lat array
REAL, DIMENSION(:,:), INTENT(IN)  :: PLON ! MNH lon array
REAL, DIMENSION(:),   INTENT(OUT) :: PAESEA  ! integrated optical thickness for sea aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAELAND ! integrated optical thickness for lan aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAEURB  ! integrated optical thickness for urb aerosol 
REAL, DIMENSION(:),   INTENT(OUT) :: PAEDES  ! integrated optical thickness for des aerosol 
!
!
!*       0.2   declarations of local variables
!
INTEGER                           ::  IINLA  ! number of lat in global clim
INTEGER, DIMENSION(:), ALLOCATABLE :: IINLON ! number of index in global clim
INTEGER                           ::  IDIM_CLIM !  dimension of clim array 
!
REAL                              ::  ZILA1  ! lat of nothern reference point of the global clim
REAL                              ::  ZILON1 ! lon of nothern reference point of the global clim
REAL                              ::  ZILA2  ! lat of southern reference point of the global clim
REAL                              ::  ZILON2 ! lon of southern reference point of the global clim
REAL                              ::  ZTIME  ! Reference time
!
REAL, DIMENSION(:), ALLOCATABLE   :: ZHORCLIM_SEA  ! int opt thick for sea class in the clim
REAL, DIMENSION(:), ALLOCATABLE   :: ZHORCLIM_LAND ! int opt thick for land class in the clim
REAL, DIMENSION(:), ALLOCATABLE   :: ZHORCLIM_URB  ! int opt thick for urb class in the clim
REAL, DIMENSION(:), ALLOCATABLE   :: ZHORCLIM_DES  ! int opt thick for des class in the clim
REAL, DIMENSION(:), ALLOCATABLE   :: ZXOUT         ! output array ( horibl)
REAL, DIMENSION(:), ALLOCATABLE   :: ZYOUT         !
!
INTEGER    :: JI,JJ,IIJ          ! loop indices
!
!------------------------------------------------------
!
!initialisation of global regular grid
!
IF (HAER=='TANR') THEN
  IINLA=32
  ALLOCATE (IINLON(IINLA))
  IINLON(:)=64
  IDIM_CLIM= SUM (IINLON(:) )
!
!reference coordinate of climatology
!
  ZILA1=87.1875
  ZILON1=2.8125
  ZILA2=-87.1875
  ZILON2=357.1875
!
  ALLOCATE (ZHORCLIM_SEA (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_LAND (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_URB (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_DES (IDIM_CLIM))
! 
!Loading of the climatology
!
  CALL INI_CLIMCEP
END IF
!
!
IF (HAER=='TEGE') THEN 
  IINLA=46
  ALLOCATE (IINLON(IINLA))
  IINLON(:)=72
  IDIM_CLIM= SUM (IINLON(:) )
!
!reference coordinate of climatology (see suecaec)
!
  ZILA1=90.
  ZILON1=0.
  ZILA2=-90.
  ZILON2=355.
!
!Loading of the climatology
!
  ALLOCATE (ZHORCLIM_SEA (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_LAND (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_URB (IDIM_CLIM))
  ALLOCATE (ZHORCLIM_DES (IDIM_CLIM))
!
  CALL SUECAEBC
  CALL SUECAEOR
  CALL SUECAESD
  CALL SUECAESS
  CALL SUECAESU
  CALL SUECAEC ( KYMD, KHOURS )
!
  DO JI = 1, IINLA ! latitude index
    DO JJ = 1, 72 
      IIJ = (JI-1) * 72 + JJ  ! reformatted array for horribl from ECMWF arrays
      ZHORCLIM_URB(IIJ) = RTAEBC(JJ,JI)
      ZHORCLIM_DES(IIJ) = RTAESD(JJ,JI)
      ZHORCLIM_SEA(IIJ) = RTAESS(JJ,JI)
      ZHORCLIM_LAND(IIJ)= RTAESU(JJ,JI) + RTAEOR(JJ,JI)
    END DO
  END DO
!
! Important :  TEGE climatology gives directly aerosols optical thickness (not the relative 
! contribution as in TANR : Weight used 
! to determine the final 3D aerosol field in radaer  must be forced to 1      
!
  RCAEOPS = 1.  !  cf module  YOEAERD 
  RCAEOPL = 1.  !
  RCAEOPU = 1.  ! 
  RCAEOPD = 1.  !
!
END IF
!
! definition of output arrays
!
ALLOCATE(ZXOUT(KDLON))
ALLOCATE(ZYOUT(KDLON))
IF(.NOT.LCARTESIAN) THEN
  DO JJ=KJB,KJE
    DO JI=KIB,KIE
      IIJ = 1 + (JI-KIB) + (KIE-KIB+1)*(JJ-KJB)
      ZXOUT(IIJ)=PLON(JI,JJ)
      ZYOUT(IIJ)=PLAT(JI,JJ)
    END DO
  END DO
ELSE 
  ZXOUT(:)=XLON0
  ZYOUT(:)=XLAT0
END IF
!
!interpolation on MNH domain with 
!
ZTIME=0.
!
CALL HORIBL( ZILA1,ZILON1,ZILA2,ZILON2,IINLA,IINLON,IDIM_CLIM,ZHORCLIM_SEA,&
     KDLON,ZXOUT,ZYOUT,PAESEA,.FALSE.,ZTIME,.FALSE.)
!
CALL HORIBL( ZILA1,ZILON1,ZILA2,ZILON2,IINLA,IINLON,IDIM_CLIM,ZHORCLIM_LAND,&
     KDLON,ZXOUT,ZYOUT,PAELAND,.TRUE.,ZTIME,.FALSE.)
!
CALL HORIBL( ZILA1,ZILON1,ZILA2,ZILON2,IINLA,IINLON,IDIM_CLIM,ZHORCLIM_URB,&
     KDLON,ZXOUT,ZYOUT,PAEURB,.TRUE.,ZTIME,.FALSE.)
!
CALL HORIBL( ZILA1,ZILON1,ZILA2,ZILON2,IINLA,IINLON,IDIM_CLIM,ZHORCLIM_DES,&
     KDLON,ZXOUT,ZYOUT,PAEDES,.TRUE.,ZTIME,.FALSE.)
!
PAESEA=MAX(0.,PAESEA)
PAELAND=MAX(0.,PAELAND)
PAEURB=MAX(0.,PAEURB)
PAEDES=MAX(0.,PAEDES)
!
DEALLOCATE (ZHORCLIM_SEA )
DEALLOCATE (ZHORCLIM_LAND)
DEALLOCATE (ZHORCLIM_URB )
DEALLOCATE (ZHORCLIM_DES )
DEALLOCATE(ZXOUT)
DEALLOCATE(ZYOUT)
IF (HAER=='TANR' .OR. HAER=='TEGE') DEALLOCATE (IINLON)
!
!
CONTAINS
!----------------------------------------------------
!
SUBROUTINE INI_CLIMCEP
!!    PURPOSE
!!    -------
!        This routine fills the climatology array in TANR case
!
! sea vector
ZHORCLIM_SEA( 1 : 512 ) = (/ &
     0.2121,    0.2109,    0.2097,    0.2084,    0.2069,    0.2054,    0.2039,    0.2024, &
     0.2009,    0.1994,    0.1980,    0.1966,    0.1954,    0.1943,    0.1934,    0.1925, &
     0.1919,    0.1914,    0.1910,    0.1908,    0.1908,    0.1908,    0.1910,    0.1914, &
     0.1918,    0.1923,    0.1929,    0.1935,    0.1942,    0.1949,    0.1956,    0.1964, &
     0.1971,    0.1978,    0.1985,    0.1993,    0.2000,    0.2007,    0.2014,    0.2021, &
     0.2029,    0.2036,    0.2044,    0.2053,    0.2061,    0.2070,    0.2079,    0.2088, &
     0.2097,    0.2106,    0.2114,    0.2123,    0.2130,    0.2137,    0.2143,    0.2148, &
     0.2151,    0.2153,    0.2154,    0.2153,    0.2150,    0.2145,    0.2138,    0.2130, &
     0.2515,    0.2468,    0.2414,    0.2355,    0.2292,    0.2227,    0.2161,    0.2096, &
     0.2032,    0.1972,    0.1917,    0.1867,    0.1823,    0.1787,    0.1759,    0.1739, &
     0.1728,    0.1725,    0.1730,    0.1743,    0.1762,    0.1788,    0.1819,    0.1853, &
     0.1890,    0.1929,    0.1967,    0.2004,    0.2039,    0.2071,    0.2099,    0.2122, &
     0.2140,    0.2153,    0.2161,    0.2165,    0.2165,    0.2163,    0.2159,    0.2154, &
     0.2150,    0.2148,    0.2149,    0.2154,    0.2164,    0.2180,    0.2201,    0.2228, &
     0.2261,    0.2298,    0.2339,    0.2382,    0.2426,    0.2469,    0.2510,    0.2546, &
     0.2577,    0.2601,    0.2617,    0.2624,    0.2621,    0.2608,    0.2586,    0.2555, &
     0.3052,    0.2948,    0.2831,    0.2705,    0.2573,    0.2438,    0.2304,    0.2173, &
     0.2049,    0.1933,    0.1829,    0.1738,    0.1663,    0.1604,    0.1562,    0.1540, &
     0.1536,    0.1551,    0.1584,    0.1635,    0.1700,    0.1779,    0.1869,    0.1966, &
     0.2067,    0.2169,    0.2269,    0.2362,    0.2447,    0.2520,    0.2580,    0.2624, &
     0.2652,    0.2664,    0.2660,    0.2642,    0.2610,    0.2568,    0.2519,    0.2467, &
     0.2415,    0.2367,    0.2328,    0.2300,    0.2288,    0.2293,    0.2316,    0.2359, &
     0.2420,    0.2497,    0.2587,    0.2687,    0.2792,    0.2898,    0.2999,    0.3091, &
     0.3169,    0.3230,    0.3271,    0.3290,    0.3286,    0.3259,    0.3210,    0.3140, &
     0.3621,    0.3441,    0.3246,    0.3039,    0.2828,    0.2616,    0.2410,    0.2212, &
     0.2028,    0.1860,    0.1711,    0.1585,    0.1483,    0.1408,    0.1362,    0.1346, &
     0.1362,    0.1409,    0.1487,    0.1594,    0.1727,    0.1883,    0.2056,    0.2242, &
     0.2433,    0.2624,    0.2808,    0.2979,    0.3132,    0.3261,    0.3363,    0.3435, &
     0.3475,    0.3483,    0.3459,    0.3405,    0.3324,    0.3222,    0.3102,    0.2973, &
     0.2841,    0.2716,    0.2605,    0.2516,    0.2456,    0.2431,    0.2445,    0.2498, &
     0.2589,    0.2714,    0.2868,    0.3043,    0.3228,    0.3415,    0.3594,    0.3755, &
     0.3889,    0.3991,    0.4055,    0.4078,    0.4060,    0.4002,    0.3906,    0.3777, &
     0.4124,    0.3858,    0.3576,    0.3288,    0.3001,    0.2720,    0.2452,    0.2200, &
     0.1968,    0.1761,    0.1581,    0.1430,    0.1313,    0.1231,    0.1189,    0.1188, &
     0.1232,    0.1321,    0.1454,    0.1631,    0.1847,    0.2097,    0.2372,    0.2663, &
     0.2962,    0.3256,    0.3538,    0.3796,    0.4024,    0.4215,    0.4363,    0.4465, &
     0.4519,    0.4523,    0.4477,    0.4385,    0.4249,    0.4076,    0.3873,    0.3650, &
     0.3419,    0.3193,    0.2986,    0.2813,    0.2686,    0.2615,    0.2608,    0.2666, &
     0.2788,    0.2968,    0.3194,    0.3454,    0.3731,    0.4009,    0.4271,    0.4502, &
     0.4689,    0.4822,    0.4894,    0.4903,    0.4850,    0.4737,    0.4573,    0.4366, &
     0.4490,    0.4136,    0.3775,    0.3417,    0.3071,    0.2742,    0.2434,    0.2150, &
     0.1895,    0.1668,    0.1474,    0.1314,    0.1192,    0.1112,    0.1080,    0.1099, &
     0.1174,    0.1309,    0.1505,    0.1758,    0.2065,    0.2417,    0.2802,    0.3206, &
     0.3615,    0.4015,    0.4392,    0.4735,    0.5033,    0.5279,    0.5469,    0.5597, &
     0.5662,    0.5663,    0.5600,    0.5474,    0.5289,    0.5050,    0.4765,    0.4448, &
     0.4112,    0.3778,    0.3465,    0.3195,    0.2989,    0.2862,    0.2827,    0.2887, &
     0.3041,    0.3276,    0.3579,    0.3926,    0.4295,    0.4661,    0.5000,    0.5290, &
     0.5513,    0.5658,    0.5715,    0.5684,    0.5569,    0.5379,    0.5126,    0.4824, &
     0.4678,    0.4246,    0.3824,    0.3421,    0.3044,    0.2697,    0.2381,    0.2097, &
     0.1845,    0.1625,    0.1437,    0.1284,    0.1168,    0.1096,    0.1076,    0.1114, &
     0.1221,    0.1400,    0.1657,    0.1988,    0.2386,    0.2839,    0.3330,    0.3842, &
     0.4353,    0.4845,    0.5301,    0.5709,    0.6059,    0.6344,    0.6560,    0.6705, &
     0.6777,    0.6777,    0.6703,    0.6555,    0.6336,    0.6049,    0.5702,    0.5305, &
     0.4878,    0.4443,    0.4027,    0.3662,    0.3375,    0.3191,    0.3125,    0.3186, &
     0.3367,    0.3656,    0.4028,    0.4454,    0.4903,    0.5342,    0.5739,    0.6067, &
     0.6304,    0.6434,    0.6451,    0.6354,    0.6153,    0.5864,    0.5507,    0.5105, &
     0.4684,    0.4191,    0.3731,    0.3313,    0.2940,    0.2611,    0.2324,    0.2075, &
     0.1860,    0.1674,    0.1516,    0.1386,    0.1287,    0.1227,    0.1218,    0.1272, &
     0.1403,    0.1621,    0.1931,    0.2331,    0.2811,    0.3356,    0.3941,    0.4541, &
     0.5131,    0.5690,    0.6197,    0.6642,    0.7015,    0.7314,    0.7537,    0.7685, &
     0.7758,    0.7758,    0.7682,    0.7529,    0.7299,    0.6990,    0.6608,    0.6161, &
     0.5669,    0.5157,    0.4659,    0.4213,    0.3855,    0.3618,    0.3523,    0.3580, &
     0.3782,    0.4111,    0.4536,    0.5021,    0.5527,    0.6012,    0.6442,    0.6781, &
     0.7006,    0.7099,    0.7051,    0.6868,    0.6564,    0.6163,    0.5696,    0.5193 &
     /)
ZHORCLIM_SEA( 513 : 1024 ) = (/ &
     0.4536,    0.4002,    0.3529,    0.3124,    0.2786,    0.2511,    0.2289,    0.2112, &
     0.1969,    0.1850,    0.1747,    0.1659,    0.1588,    0.1544,    0.1543,    0.1605, &
     0.1750,    0.1993,    0.2342,    0.2796,    0.3341,    0.3956,    0.4609,    0.5271, &
     0.5909,    0.6499,    0.7023,    0.7471,    0.7838,    0.8126,    0.8337,    0.8475, &
     0.8543,    0.8543,    0.8474,    0.8331,    0.8111,    0.7810,    0.7426,    0.6967, &
     0.6447,    0.5894,    0.5344,    0.4842,    0.4432,    0.4153,    0.4031,    0.4077, &
     0.4284,    0.4629,    0.5078,    0.5589,    0.6117,    0.6618,    0.7052,    0.7380, &
     0.7575,    0.7615,    0.7493,    0.7215,    0.6802,    0.6287,    0.5711,    0.5115, &
     0.4296,    0.3735,    0.3265,    0.2892,    0.2613,    0.2419,    0.2295,    0.2224, &
     0.2189,    0.2169,    0.2150,    0.2124,    0.2093,    0.2069,    0.2074,    0.2135, &
     0.2280,    0.2530,    0.2897,    0.3380,    0.3963,    0.4617,    0.5306,    0.5992, &
     0.6641,    0.7228,    0.7736,    0.8158,    0.8495,    0.8752,    0.8937,    0.9056, &
     0.9115,    0.9116,    0.9058,    0.8938,    0.8746,    0.8477,    0.8123,    0.7686, &
     0.7179,    0.6625,    0.6063,    0.5538,    0.5100,    0.4791,    0.4642,    0.4663, &
     0.4848,    0.5175,    0.5606,    0.6101,    0.6613,    0.7098,    0.7513,    0.7817, &
     0.7977,    0.7967,    0.7777,    0.7413,    0.6901,    0.6283,    0.5609,    0.4932, &
     0.4047,    0.3464,    0.3001,    0.2666,    0.2456,    0.2358,    0.2354,    0.2419, &
     0.2522,    0.2633,    0.2724,    0.2781,    0.2802,    0.2803,    0.2811,    0.2861, &
     0.2989,    0.3225,    0.3583,    0.4064,    0.4648,    0.5303,    0.5988,    0.6662, &
     0.7288,    0.7840,    0.8306,    0.8684,    0.8977,    0.9195,    0.9349,    0.9446, &
     0.9494,    0.9497,    0.9454,    0.9360,    0.9207,    0.8985,    0.8684,    0.8300, &
     0.7840,    0.7324,    0.6787,    0.6274,    0.5832,    0.5506,    0.5325,    0.5302, &
     0.5432,    0.5694,    0.6060,    0.6491,    0.6948,    0.7389,    0.7772,    0.8053, &
     0.8191,    0.8154,    0.7923,    0.7502,    0.6920,    0.6222,    0.5471,    0.4728, &
     0.3876,    0.3270,    0.2806,    0.2500,    0.2351,    0.2351,    0.2477,    0.2695, &
     0.2958,    0.3221,    0.3443,    0.3599,    0.3683,    0.3712,    0.3721,    0.3752, &
     0.3848,    0.4046,    0.4365,    0.4806,    0.5352,    0.5968,    0.6609,    0.7234, &
     0.7808,    0.8305,    0.8718,    0.9045,    0.9295,    0.9478,    0.9604,    0.9683, &
     0.9723,    0.9728,    0.9699,    0.9633,    0.9520,    0.9351,    0.9113,    0.8800, &
     0.8411,    0.7963,    0.7483,    0.7010,    0.6587,    0.6252,    0.6033,    0.5942, &
     0.5979,    0.6131,    0.6379,    0.6700,    0.7066,    0.7444,    0.7793,    0.8066, &
     0.8212,    0.8186,    0.7960,    0.7530,    0.6921,    0.6185,    0.5387,    0.4597, &
     0.3861,    0.3224,    0.2743,    0.2444,    0.2340,    0.2425,    0.2676,    0.3048, &
     0.3478,    0.3901,    0.4260,    0.4520,    0.4672,    0.4733,    0.4742,    0.4748, &
     0.4799,    0.4938,    0.5188,    0.5555,    0.6022,    0.6557,    0.7119,    0.7667, &
     0.8169,    0.8604,    0.8964,    0.9251,    0.9469,    0.9629,    0.9739,    0.9809, &
     0.9846,    0.9856,    0.9840,    0.9797,    0.9720,    0.9601,    0.9426,    0.9188, &
     0.8882,    0.8518,    0.8115,    0.7702,    0.7312,    0.6975,    0.6711,    0.6532, &
     0.6442,    0.6441,    0.6528,    0.6700,    0.6947,    0.7250,    0.7571,    0.7859, &
     0.8050,    0.8084,    0.7919,    0.7538,    0.6960,    0.6234,    0.5427,    0.4613, &
     0.4043,    0.3369,    0.2852,    0.2537,    0.2452,    0.2601,    0.2959,    0.3468, &
     0.4052,    0.4624,    0.5112,    0.5471,    0.5688,    0.5781,    0.5790,    0.5768, &
     0.5767,    0.5831,    0.5989,    0.6250,    0.6604,    0.7025,    0.7477,    0.7928, &
     0.8350,    0.8726,    0.9048,    0.9312,    0.9520,    0.9677,    0.9788,    0.9860, &
     0.9901,    0.9918,    0.9914,    0.9889,    0.9842,    0.9763,    0.9644,    0.9475, &
     0.9251,    0.8973,    0.8653,    0.8308,    0.7960,    0.7623,    0.7312,    0.7033, &
     0.6795,    0.6615,    0.6511,    0.6507,    0.6618,    0.6839,    0.7140,    0.7464, &
     0.7735,    0.7877,    0.7827,    0.7555,    0.7067,    0.6403,    0.5629,    0.4818, &
     0.4420,    0.3709,    0.3146,    0.2796,    0.2706,    0.2891,    0.3326,    0.3944, &
     0.4650,    0.5342,    0.5934,    0.6374,    0.6645,    0.6766,    0.6777,    0.6730, &
     0.6675,    0.6657,    0.6708,    0.6841,    0.7056,    0.7337,    0.7660,    0.8004, &
     0.8348,    0.8677,    0.8979,    0.9246,    0.9469,    0.9644,    0.9773,    0.9861, &
     0.9914,    0.9941,    0.9948,    0.9939,    0.9912,    0.9864,    0.9788,    0.9676, &
     0.9522,    0.9323,    0.9081,    0.8804,    0.8497,    0.8165,    0.7808,    0.7429, &
     0.7042,    0.6676,    0.6374,    0.6186,    0.6154,    0.6291,    0.6576,    0.6950, &
     0.7324,    0.7605,    0.7711,    0.7593,    0.7244,    0.6690,    0.5987,    0.5205, &
     0.4951,    0.4212,    0.3603,    0.3211,    0.3099,    0.3295,    0.3773,    0.4458, &
     0.5243,    0.6012,    0.6673,    0.7166,    0.7475,    0.7618,    0.7631,    0.7563, &
     0.7459,    0.7360,    0.7298,    0.7295,    0.7357,    0.7486,    0.7672,    0.7907, &
     0.8179,    0.8476,    0.8780,    0.9072,    0.9334,    0.9549,    0.9712,    0.9826, &
     0.9898,    0.9940,    0.9959,    0.9962,    0.9950,    0.9924,    0.9879,    0.9810, &
     0.9710,    0.9574,    0.9398,    0.9180,    0.8912,    0.8587,    0.8194,    0.7730, &
     0.7211,    0.6677,    0.6192,    0.5832,    0.5663,    0.5719,    0.5989,    0.6412, &
     0.6892,    0.7319,    0.7597,    0.7657,    0.7474,    0.7060,    0.6457,    0.5728 &
     /)
ZHORCLIM_SEA( 1025 : 1536 ) = (/ &
     0.5569,    0.4822,    0.4184,    0.3756,    0.3618,    0.3805,    0.4292,    0.4999, &
     0.5812,    0.6610,    0.7296,    0.7812,    0.8139,    0.8295,    0.8311,    0.8227, &
     0.8081,    0.7909,    0.7742,    0.7604,    0.7516,    0.7493,    0.7544,    0.7676, &
     0.7887,    0.8165,    0.8487,    0.8821,    0.9137,    0.9406,    0.9616,    0.9765, &
     0.9863,    0.9923,    0.9955,    0.9968,    0.9968,    0.9956,    0.9932,    0.9892, &
     0.9831,    0.9742,    0.9616,    0.9444,    0.9211,    0.8896,    0.8480,    0.7955, &
     0.7337,    0.6674,    0.6045,    0.5544,    0.5260,    0.5243,    0.5494,    0.5953, &
     0.6521,    0.7078,    0.7513,    0.7744,    0.7728,    0.7462,    0.6974,    0.6317, &
     0.6205,    0.5480,    0.4840,    0.4398,    0.4240,    0.4405,    0.4871,    0.5557, &
     0.6349,    0.7128,    0.7799,    0.8306,    0.8632,    0.8790,    0.8807,    0.8712, &
     0.8535,    0.8303,    0.8045,    0.7790,    0.7567,    0.7408,    0.7338,    0.7380, &
     0.7539,    0.7805,    0.8150,    0.8532,    0.8906,    0.9234,    0.9495,    0.9685, &
     0.9813,    0.9893,    0.9939,    0.9963,    0.9972,    0.9971,    0.9960,    0.9938, &
     0.9902,    0.9844,    0.9753,    0.9615,    0.9408,    0.9107,    0.8685,    0.8130, &
     0.7456,    0.6715,    0.5994,    0.5401,    0.5036,    0.4962,    0.5189,    0.5664, &
     0.6286,    0.6933,    0.7486,    0.7854,    0.7981,    0.7849,    0.7474,    0.6902, &
     0.6806,    0.6136,    0.5530,    0.5102,    0.4938,    0.5077,    0.5498,    0.6127, &
     0.6856,    0.7575,    0.8196,    0.8667,    0.8973,    0.9123,    0.9136,    0.9034, &
     0.8836,    0.8562,    0.8237,    0.7892,    0.7565,    0.7298,    0.7133,    0.7101, &
     0.7217,    0.7472,    0.7834,    0.8254,    0.8676,    0.9055,    0.9363,    0.9593, &
     0.9750,    0.9852,    0.9913,    0.9948,    0.9965,    0.9971,    0.9969,    0.9959, &
     0.9937,    0.9898,    0.9828,    0.9713,    0.9527,    0.9241,    0.8827,    0.8271, &
     0.7585,    0.6823,    0.6071,    0.5442,    0.5037,    0.4929,    0.5131,    0.5597, &
     0.6232,    0.6917,    0.7534,    0.7987,    0.8213,    0.8186,    0.7913,    0.7430, &
     0.7344,    0.6757,    0.6221,    0.5836,    0.5682,    0.5794,    0.6155,    0.6700, &
     0.7334,    0.7961,    0.8505,    0.8920,    0.9192,    0.9325,    0.9331,    0.9224, &
     0.9015,    0.8719,    0.8357,    0.7959,    0.7568,    0.7234,    0.7005,    0.6921, &
     0.7001,    0.7240,    0.7602,    0.8037,    0.8484,    0.8894,    0.9235,    0.9495, &
     0.9678,    0.9800,    0.9876,    0.9921,    0.9946,    0.9959,    0.9962,    0.9959, &
     0.9945,    0.9914,    0.9857,    0.9754,    0.9583,    0.9314,    0.8921,    0.8388, &
     0.7731,    0.6997,    0.6272,    0.5660,    0.5260,    0.5140,    0.5318,    0.5754, &
     0.6362,    0.7032,    0.7654,    0.8135,    0.8413,    0.8456,    0.8267,    0.7876, &
     0.7799,    0.7320,    0.6877,    0.6557,    0.6427,    0.6515,    0.6808,    0.7253, &
     0.7773,    0.8289,    0.8740,    0.9086,    0.9313,    0.9421,    0.9417,    0.9309, &
     0.9102,    0.8806,    0.8438,    0.8029,    0.7619,    0.7261,    0.7005,    0.6892, &
     0.6944,    0.7156,    0.7497,    0.7916,    0.8358,    0.8771,    0.9122,    0.9396, &
     0.9596,    0.9732,    0.9821,    0.9876,    0.9909,    0.9927,    0.9935,    0.9935, &
     0.9924,    0.9897,    0.9844,    0.9748,    0.9588,    0.9336,    0.8971,    0.8482, &
     0.7883,    0.7218,    0.6565,    0.6014,    0.5652,    0.5542,    0.5697,    0.6085, &
     0.6633,    0.7244,    0.7821,    0.8281,    0.8567,    0.8649,    0.8527,    0.8226, &
     0.8152,    0.7790,    0.7454,    0.7211,    0.7112,    0.7180,    0.7404,    0.7745, &
     0.8145,    0.8544,    0.8895,    0.9165,    0.9342,    0.9423,    0.9409,    0.9304, &
     0.9110,    0.8835,    0.8493,    0.8110,    0.7725,    0.7384,    0.7135,    0.7015, &
     0.7046,    0.7223,    0.7522,    0.7898,    0.8301,    0.8687,    0.9023,    0.9293, &
     0.9495,    0.9639,    0.9737,    0.9800,    0.9840,    0.9863,    0.9874,    0.9875, &
     0.9865,    0.9838,    0.9785,    0.9692,    0.9539,    0.9306,    0.8976,    0.8542, &
     0.8020,    0.7450,    0.6896,    0.6432,    0.6131,    0.6040,    0.6173,    0.6501, &
     0.6965,    0.7487,    0.7984,    0.8390,    0.8654,    0.8753,    0.8684,    0.8470, &
     0.8369,    0.8122,    0.7892,    0.7726,    0.7660,    0.7711,    0.7871,    0.8115, &
     0.8401,    0.8689,    0.8942,    0.9139,    0.9266,    0.9319,    0.9297,    0.9201, &
     0.9032,    0.8794,    0.8502,    0.8175,    0.7847,    0.7556,    0.7340,    0.7232, &
     0.7249,    0.7389,    0.7632,    0.7945,    0.8286,    0.8620,    0.8918,    0.9166, &
     0.9358,    0.9501,    0.9601,    0.9669,    0.9714,    0.9741,    0.9754,    0.9756, &
     0.9744,    0.9715,    0.9659,    0.9567,    0.9421,    0.9208,    0.8917,    0.8545, &
     0.8110,    0.7643,    0.7197,    0.6829,    0.6593,    0.6525,    0.6631,    0.6893, &
     0.7261,    0.7678,    0.8080,    0.8413,    0.8640,    0.8742,    0.8717,    0.8582, &
     0.8405,    0.8258,    0.8120,    0.8022,    0.7987,    0.8025,    0.8133,    0.8294, &
     0.8483,    0.8673,    0.8842,    0.8972,    0.9054,    0.9083,    0.9057,    0.8974, &
     0.8837,    0.8648,    0.8419,    0.8166,    0.7913,    0.7689,    0.7524,    0.7440, &
     0.7451,    0.7556,    0.7741,    0.7982,    0.8251,    0.8520,    0.8767,    0.8980, &
     0.9151,    0.9284,    0.9381,    0.9450,    0.9497,    0.9526,    0.9540,    0.9541, &
     0.9527,    0.9494,    0.9435,    0.9343,    0.9206,    0.9015,    0.8764,    0.8456, &
     0.8106,    0.7740,    0.7396,    0.7116,    0.6938,    0.6885,    0.6962,    0.7154, &
     0.7426,    0.7737,    0.8040,    0.8299,    0.8484,    0.8583,    0.8593,    0.8526 &
     /)
ZHORCLIM_SEA( 1537 : 2048 ) = (/ &
     0.8214,    0.8147,    0.8081,    0.8037,    0.8026,    0.8056,    0.8124,    0.8223, &
     0.8337,    0.8451,    0.8552,    0.8629,    0.8674,    0.8684,    0.8657,    0.8591, &
     0.8488,    0.8351,    0.8188,    0.8011,    0.7836,    0.7684,    0.7573,    0.7519, &
     0.7531,    0.7608,    0.7743,    0.7919,    0.8119,    0.8323,    0.8516,    0.8688, &
     0.8832,    0.8948,    0.9038,    0.9104,    0.9150,    0.9179,    0.9192,    0.9191, &
     0.9173,    0.9136,    0.9076,    0.8985,    0.8859,    0.8691,    0.8480,    0.8232, &
     0.7958,    0.7680,    0.7423,    0.7215,    0.7080,    0.7034,    0.7080,    0.7205, &
     0.7388,    0.7602,    0.7817,    0.8006,    0.8151,    0.8242,    0.8277,    0.8263, &
     0.7776,    0.7763,    0.7748,    0.7741,    0.7748,    0.7773,    0.7816,    0.7872, &
     0.7935,    0.7996,    0.8049,    0.8088,    0.8107,    0.8105,    0.8080,    0.8032, &
     0.7962,    0.7872,    0.7768,    0.7659,    0.7553,    0.7464,    0.7403,    0.7378, &
     0.7395,    0.7455,    0.7552,    0.7677,    0.7820,    0.7968,    0.8113,    0.8246, &
     0.8362,    0.8459,    0.8537,    0.8596,    0.8639,    0.8665,    0.8675,    0.8671, &
     0.8650,    0.8610,    0.8549,    0.8463,    0.8349,    0.8205,    0.8033,    0.7838, &
     0.7629,    0.7421,    0.7231,    0.7076,    0.6972,    0.6926,    0.6940,    0.7008, &
     0.7117,    0.7250,    0.7390,    0.7520,    0.7629,    0.7708,    0.7757,    0.7777, &
     0.7115,    0.7132,    0.7146,    0.7160,    0.7177,    0.7200,    0.7228,    0.7260, &
     0.7292,    0.7321,    0.7345,    0.7360,    0.7365,    0.7358,    0.7339,    0.7307, &
     0.7265,    0.7213,    0.7157,    0.7099,    0.7047,    0.7007,    0.6984,    0.6984, &
     0.7008,    0.7058,    0.7129,    0.7219,    0.7320,    0.7427,    0.7532,    0.7632, &
     0.7722,    0.7800,    0.7864,    0.7914,    0.7950,    0.7971,    0.7978,    0.7971, &
     0.7947,    0.7907,    0.7848,    0.7769,    0.7670,    0.7551,    0.7414,    0.7264, &
     0.7107,    0.6954,    0.6813,    0.6697,    0.6611,    0.6563,    0.6552,    0.6577, &
     0.6629,    0.6701,    0.6782,    0.6865,    0.6940,    0.7004,    0.7054,    0.7090, &
     0.6301,    0.6328,    0.6353,    0.6375,    0.6396,    0.6417,    0.6437,    0.6456, &
     0.6473,    0.6487,    0.6497,    0.6502,    0.6502,    0.6495,    0.6484,    0.6467, &
     0.6446,    0.6423,    0.6400,    0.6379,    0.6363,    0.6355,    0.6358,    0.6373, &
     0.6403,    0.6446,    0.6502,    0.6568,    0.6640,    0.6717,    0.6793,    0.6866, &
     0.6933,    0.6992,    0.7041,    0.7080,    0.7108,    0.7123,    0.7126,    0.7116, &
     0.7092,    0.7054,    0.7001,    0.6934,    0.6852,    0.6758,    0.6653,    0.6541, &
     0.6426,    0.6314,    0.6211,    0.6122,    0.6052,    0.6003,    0.5977,    0.5972, &
     0.5987,    0.6016,    0.6056,    0.6101,    0.6147,    0.6192,    0.6233,    0.6269, &
     0.5447,    0.5472,    0.5494,    0.5515,    0.5534,    0.5551,    0.5566,    0.5580, &
     0.5591,    0.5599,    0.5605,    0.5609,    0.5610,    0.5609,    0.5605,    0.5601, &
     0.5597,    0.5592,    0.5590,    0.5590,    0.5595,    0.5605,    0.5621,    0.5644, &
     0.5675,    0.5712,    0.5755,    0.5804,    0.5855,    0.5909,    0.5962,    0.6012, &
     0.6059,    0.6101,    0.6135,    0.6162,    0.6181,    0.6190,    0.6189,    0.6178, &
     0.6156,    0.6124,    0.6081,    0.6029,    0.5967,    0.5897,    0.5822,    0.5742, &
     0.5662,    0.5583,    0.5510,    0.5444,    0.5388,    0.5345,    0.5313,    0.5295, &
     0.5288,    0.5291,    0.5303,    0.5322,    0.5344,    0.5370,    0.5396,    0.5422, &
     0.4687,    0.4701,    0.4716,    0.4729,    0.4742,    0.4755,    0.4766,    0.4776, &
     0.4784,    0.4792,    0.4799,    0.4804,    0.4809,    0.4814,    0.4818,    0.4823, &
     0.4828,    0.4835,    0.4843,    0.4854,    0.4867,    0.4883,    0.4902,    0.4925, &
     0.4951,    0.4980,    0.5011,    0.5044,    0.5078,    0.5112,    0.5146,    0.5177, &
     0.5206,    0.5231,    0.5251,    0.5267,    0.5277,    0.5280,    0.5277,    0.5267, &
     0.5250,    0.5227,    0.5197,    0.5161,    0.5120,    0.5075,    0.5026,    0.4976, &
     0.4925,    0.4874,    0.4827,    0.4782,    0.4743,    0.4709,    0.4681,    0.4660, &
     0.4645,    0.4637,    0.4634,    0.4635,    0.4641,    0.4650,    0.4661,    0.4673, &
     0.4146,    0.4151,    0.4156,    0.4162,    0.4168,    0.4174,    0.4181,    0.4187, &
     0.4194,    0.4201,    0.4207,    0.4214,    0.4221,    0.4228,    0.4236,    0.4245, &
     0.4254,    0.4263,    0.4274,    0.4286,    0.4299,    0.4313,    0.4328,    0.4344, &
     0.4361,    0.4378,    0.4397,    0.4415,    0.4433,    0.4450,    0.4467,    0.4482, &
     0.4495,    0.4506,    0.4515,    0.4521,    0.4524,    0.4524,    0.4521,    0.4514, &
     0.4504,    0.4490,    0.4474,    0.4455,    0.4434,    0.4410,    0.4385,    0.4360, &
     0.4333,    0.4307,    0.4282,    0.4258,    0.4235,    0.4215,    0.4197,    0.4181, &
     0.4168,    0.4157,    0.4149,    0.4144,    0.4141,    0.4140,    0.4140,    0.4143, &
     0.3917,    0.3917,    0.3917,    0.3918,    0.3919,    0.3920,    0.3922,    0.3925, &
     0.3927,    0.3931,    0.3934,    0.3938,    0.3942,    0.3946,    0.3950,    0.3955, &
     0.3960,    0.3965,    0.3971,    0.3976,    0.3982,    0.3988,    0.3994,    0.3999, &
     0.4005,    0.4011,    0.4016,    0.4021,    0.4026,    0.4030,    0.4034,    0.4037, &
     0.4040,    0.4042,    0.4044,    0.4044,    0.4044,    0.4044,    0.4042,    0.4040, &
     0.4036,    0.4032,    0.4028,    0.4023,    0.4017,    0.4011,    0.4004,    0.3998, &
     0.3991,    0.3984,    0.3977,    0.3970,    0.3963,    0.3956,    0.3950,    0.3945, &
     0.3939,    0.3935,    0.3930,    0.3927,    0.3924,    0.3921,    0.3919,    0.3918 &
     /)
!
! land 
ZHORCLIM_LAND( 1 : 512 ) = (/ &
     0.2452,    0.2470,    0.2489,    0.2507,    0.2525,    0.2543,    0.2559,    0.2574, &
     0.2588,    0.2601,    0.2612,    0.2621,    0.2628,    0.2633,    0.2637,    0.2638, &
     0.2637,    0.2634,    0.2630,    0.2623,    0.2615,    0.2606,    0.2595,    0.2582, &
     0.2569,    0.2555,    0.2541,    0.2525,    0.2510,    0.2494,    0.2478,    0.2463, &
     0.2448,    0.2433,    0.2418,    0.2405,    0.2391,    0.2379,    0.2367,    0.2356, &
     0.2346,    0.2337,    0.2328,    0.2321,    0.2314,    0.2309,    0.2305,    0.2301, &
     0.2299,    0.2298,    0.2299,    0.2301,    0.2304,    0.2309,    0.2315,    0.2323, &
     0.2332,    0.2343,    0.2355,    0.2369,    0.2383,    0.2399,    0.2416,    0.2434, &
     0.2485,    0.2551,    0.2618,    0.2685,    0.2751,    0.2815,    0.2874,    0.2929, &
     0.2978,    0.3021,    0.3056,    0.3084,    0.3103,    0.3115,    0.3119,    0.3115, &
     0.3103,    0.3084,    0.3059,    0.3028,    0.2991,    0.2951,    0.2906,    0.2859, &
     0.2809,    0.2759,    0.2708,    0.2658,    0.2609,    0.2562,    0.2518,    0.2476, &
     0.2438,    0.2403,    0.2372,    0.2344,    0.2320,    0.2298,    0.2279,    0.2262, &
     0.2247,    0.2233,    0.2219,    0.2206,    0.2193,    0.2179,    0.2166,    0.2153, &
     0.2140,    0.2127,    0.2117,    0.2108,    0.2103,    0.2102,    0.2106,    0.2115, &
     0.2130,    0.2152,    0.2181,    0.2217,    0.2259,    0.2308,    0.2363,    0.2422, &
     0.2563,    0.2688,    0.2815,    0.2943,    0.3066,    0.3184,    0.3293,    0.3390, &
     0.3475,    0.3545,    0.3601,    0.3640,    0.3664,    0.3672,    0.3666,    0.3645, &
     0.3611,    0.3564,    0.3507,    0.3440,    0.3364,    0.3281,    0.3193,    0.3101, &
     0.3007,    0.2912,    0.2819,    0.2729,    0.2644,    0.2566,    0.2495,    0.2432, &
     0.2379,    0.2336,    0.2301,    0.2276,    0.2259,    0.2250,    0.2246,    0.2246, &
     0.2248,    0.2251,    0.2252,    0.2251,    0.2245,    0.2233,    0.2215,    0.2191, &
     0.2162,    0.2129,    0.2093,    0.2057,    0.2024,    0.1997,    0.1977,    0.1969, &
     0.1974,    0.1994,    0.2030,    0.2083,    0.2152,    0.2237,    0.2335,    0.2445, &
     0.2681,    0.2870,    0.3062,    0.3252,    0.3434,    0.3604,    0.3758,    0.3892, &
     0.4003,    0.4091,    0.4154,    0.4193,    0.4208,    0.4200,    0.4172,    0.4124, &
     0.4060,    0.3979,    0.3885,    0.3777,    0.3660,    0.3532,    0.3398,    0.3259, &
     0.3117,    0.2976,    0.2838,    0.2707,    0.2585,    0.2475,    0.2380,    0.2300, &
     0.2237,    0.2192,    0.2165,    0.2154,    0.2159,    0.2177,    0.2206,    0.2243, &
     0.2285,    0.2326,    0.2364,    0.2394,    0.2413,    0.2417,    0.2405,    0.2376, &
     0.2330,    0.2270,    0.2199,    0.2122,    0.2045,    0.1974,    0.1915,    0.1874, &
     0.1856,    0.1864,    0.1901,    0.1968,    0.2064,    0.2187,    0.2334,    0.2500, &
     0.2819,    0.3071,    0.3324,    0.3570,    0.3801,    0.4012,    0.4197,    0.4353, &
     0.4476,    0.4565,    0.4620,    0.4643,    0.4636,    0.4601,    0.4542,    0.4462, &
     0.4364,    0.4249,    0.4120,    0.3977,    0.3822,    0.3655,    0.3479,    0.3296, &
     0.3110,    0.2924,    0.2744,    0.2573,    0.2417,    0.2279,    0.2161,    0.2068, &
     0.2000,    0.1958,    0.1942,    0.1952,    0.1985,    0.2039,    0.2112,    0.2198, &
     0.2292,    0.2388,    0.2480,    0.2560,    0.2621,    0.2656,    0.2662,    0.2636, &
     0.2578,    0.2491,    0.2380,    0.2254,    0.2123,    0.1997,    0.1887,    0.1803, &
     0.1754,    0.1746,    0.1782,    0.1864,    0.1989,    0.2154,    0.2352,    0.2577, &
     0.2964,    0.3269,    0.3568,    0.3853,    0.4115,    0.4348,    0.4545,    0.4703, &
     0.4819,    0.4893,    0.4926,    0.4920,    0.4880,    0.4810,    0.4717,    0.4605, &
     0.4476,    0.4334,    0.4178,    0.4009,    0.3826,    0.3630,    0.3421,    0.3202, &
     0.2978,    0.2755,    0.2538,    0.2334,    0.2149,    0.1989,    0.1856,    0.1753, &
     0.1683,    0.1647,    0.1643,    0.1672,    0.1733,    0.1822,    0.1937,    0.2074, &
     0.2225,    0.2383,    0.2538,    0.2678,    0.2794,    0.2874,    0.2910,    0.2896, &
     0.2832,    0.2721,    0.2571,    0.2393,    0.2204,    0.2020,    0.1856,    0.1729, &
     0.1649,    0.1627,    0.1666,    0.1767,    0.1925,    0.2135,    0.2386,    0.2666, &
     0.3111,    0.3451,    0.3777,    0.4078,    0.4346,    0.4576,    0.4762,    0.4901, &
     0.4993,    0.5037,    0.5035,    0.4994,    0.4917,    0.4813,    0.4688,    0.4549, &
     0.4399,    0.4239,    0.4069,    0.3885,    0.3686,    0.3469,    0.3236,    0.2991, &
     0.2738,    0.2485,    0.2241,    0.2015,    0.1812,    0.1638,    0.1498,    0.1393, &
     0.1325,    0.1294,    0.1301,    0.1344,    0.1424,    0.1540,    0.1689,    0.1866, &
     0.2067,    0.2282,    0.2498,    0.2702,    0.2877,    0.3008,    0.3081,    0.3087, &
     0.3024,    0.2895,    0.2711,    0.2487,    0.2246,    0.2008,    0.1797,    0.1633, &
     0.1531,    0.1502,    0.1553,    0.1681,    0.1879,    0.2137,    0.2438,    0.2769, &
     0.3265,    0.3623,    0.3951,    0.4242,    0.4490,    0.4690,    0.4841,    0.4943, &
     0.4994,    0.4997,    0.4956,    0.4877,    0.4768,    0.4635,    0.4489,    0.4334, &
     0.4174,    0.4008,    0.3834,    0.3645,    0.3438,    0.3210,    0.2960,    0.2695, &
     0.2422,    0.2150,    0.1891,    0.1654,    0.1446,    0.1272,    0.1134,    0.1034, &
     0.0971,    0.0946,    0.0959,    0.1009,    0.1098,    0.1226,    0.1392,    0.1595, &
     0.1829,    0.2086,    0.2352,    0.2610,    0.2839,    0.3019,    0.3131,    0.3162, &
     0.3107,    0.2969,    0.2761,    0.2504,    0.2223,    0.1946,    0.1701,    0.1513, &
     0.1402,    0.1380,    0.1452,    0.1616,    0.1861,    0.2169,    0.2520,    0.2892 &
     /)
!
ZHORCLIM_LAND( 513 : 1024 ) = (/ &
     0.3434,    0.3791,    0.4102,    0.4360,    0.4563,    0.4711,    0.4807,    0.4852, &
     0.4850,    0.4805,    0.4724,    0.4611,    0.4477,    0.4329,    0.4174,    0.4017, &
     0.3859,    0.3699,    0.3529,    0.3343,    0.3132,    0.2896,    0.2635,    0.2356, &
     0.2071,    0.1791,    0.1529,    0.1295,    0.1094,    0.0931,    0.0804,    0.0715, &
     0.0661,    0.0641,    0.0656,    0.0706,    0.0792,    0.0918,    0.1085,    0.1294, &
     0.1541,    0.1819,    0.2114,    0.2408,    0.2678,    0.2897,    0.3044,    0.3102, &
     0.3063,    0.2929,    0.2714,    0.2443,    0.2142,    0.1846,    0.1585,    0.1389, &
     0.1281,    0.1276,    0.1379,    0.1585,    0.1879,    0.2239,    0.2636,    0.3043, &
     0.3611,    0.3957,    0.4236,    0.4446,    0.4589,    0.4669,    0.4693,    0.4668, &
     0.4603,    0.4505,    0.4383,    0.4244,    0.4095,    0.3944,    0.3796,    0.3651, &
     0.3509,    0.3363,    0.3204,    0.3023,    0.2812,    0.2569,    0.2300,    0.2013, &
     0.1724,    0.1447,    0.1194,    0.0973,    0.0790,    0.0645,    0.0537,    0.0462, &
     0.0418,    0.0403,    0.0417,    0.0460,    0.0536,    0.0648,    0.0801,    0.0997, &
     0.1236,    0.1512,    0.1813,    0.2121,    0.2411,    0.2657,    0.2833,    0.2918, &
     0.2904,    0.2792,    0.2594,    0.2334,    0.2040,    0.1746,    0.1487,    0.1295, &
     0.1196,    0.1210,    0.1343,    0.1590,    0.1932,    0.2339,    0.2777,    0.3211, &
     0.3772,    0.4101,    0.4345,    0.4503,    0.4579,    0.4584,    0.4529,    0.4427, &
     0.4292,    0.4137,    0.3975,    0.3813,    0.3660,    0.3519,    0.3390,    0.3271, &
     0.3154,    0.3030,    0.2887,    0.2714,    0.2505,    0.2260,    0.1988,    0.1701, &
     0.1416,    0.1150,    0.0914,    0.0715,    0.0554,    0.0431,    0.0341,    0.0281, &
     0.0247,    0.0235,    0.0246,    0.0281,    0.0342,    0.0434,    0.0563,    0.0733, &
     0.0946,    0.1200,    0.1485,    0.1784,    0.2075,    0.2332,    0.2529,    0.2646, &
     0.2671,    0.2603,    0.2449,    0.2230,    0.1970,    0.1700,    0.1455,    0.1270, &
     0.1177,    0.1200,    0.1351,    0.1625,    0.2001,    0.2445,    0.2915,    0.3369, &
     0.3872,    0.4187,    0.4401,    0.4514,    0.4531,    0.4465,    0.4332,    0.4152, &
     0.3945,    0.3731,    0.3528,    0.3348,    0.3196,    0.3072,    0.2971,    0.2886, &
     0.2803,    0.2708,    0.2588,    0.2430,    0.2230,    0.1991,    0.1724,    0.1445, &
     0.1174,    0.0926,    0.0710,    0.0533,    0.0394,    0.0289,    0.0215,    0.0166, &
     0.0139,    0.0129,    0.0137,    0.0161,    0.0206,    0.0276,    0.0377,    0.0514, &
     0.0692,    0.0910,    0.1161,    0.1434,    0.1709,    0.1964,    0.2178,    0.2332, &
     0.2412,    0.2413,    0.2335,    0.2188,    0.1988,    0.1759,    0.1535,    0.1351, &
     0.1247,    0.1256,    0.1398,    0.1672,    0.2059,    0.2519,    0.3005,    0.3469, &
     0.3865,    0.4172,    0.4370,    0.4454,    0.4428,    0.4305,    0.4105,    0.3852, &
     0.3577,    0.3306,    0.3064,    0.2866,    0.2716,    0.2611,    0.2543,    0.2495, &
     0.2452,    0.2394,    0.2305,    0.2174,    0.1995,    0.1775,    0.1528,    0.1270, &
     0.1020,    0.0792,    0.0595,    0.0434,    0.0308,    0.0213,    0.0147,    0.0103, &
     0.0079,    0.0069,    0.0072,    0.0088,    0.0119,    0.0168,    0.0242,    0.0346, &
     0.0484,    0.0659,    0.0867,    0.1102,    0.1350,    0.1596,    0.1826,    0.2023, &
     0.2174,    0.2267,    0.2292,    0.2245,    0.2128,    0.1953,    0.1748,    0.1553, &
     0.1416,    0.1380,    0.1476,    0.1714,    0.2075,    0.2522,    0.3003,    0.3465, &
     0.3720,    0.4027,    0.4223,    0.4298,    0.4251,    0.4092,    0.3842,    0.3532, &
     0.3199,    0.2878,    0.2601,    0.2385,    0.2236,    0.2149,    0.2110,    0.2100, &
     0.2099,    0.2084,    0.2038,    0.1948,    0.1810,    0.1629,    0.1417,    0.1192, &
     0.0969,    0.0760,    0.0575,    0.0418,    0.0291,    0.0194,    0.0124,    0.0079, &
     0.0052,    0.0040,    0.0038,    0.0047,    0.0065,    0.0098,    0.0148,    0.0222, &
     0.0323,    0.0455,    0.0618,    0.0811,    0.1027,    0.1262,    0.1506,    0.1750, &
     0.1982,    0.2183,    0.2330,    0.2403,    0.2383,    0.2269,    0.2083,    0.1864, &
     0.1670,    0.1558,    0.1572,    0.1733,    0.2031,    0.2431,    0.2880,    0.3326, &
     0.3434,    0.3744,    0.3952,    0.4038,    0.3992,    0.3820,    0.3543,    0.3196, &
     0.2821,    0.2464,    0.2159,    0.1928,    0.1779,    0.1704,    0.1688,    0.1712, &
     0.1752,    0.1785,    0.1793,    0.1761,    0.1683,    0.1562,    0.1404,    0.1224, &
     0.1031,    0.0837,    0.0651,    0.0483,    0.0338,    0.0224,    0.0139,    0.0082, &
     0.0048,    0.0030,    0.0023,    0.0025,    0.0035,    0.0054,    0.0087,    0.0135, &
     0.0205,    0.0299,    0.0421,    0.0574,    0.0760,    0.0981,    0.1238,    0.1528, &
     0.1840,    0.2152,    0.2426,    0.2623,    0.2706,    0.2657,    0.2487,    0.2238, &
     0.1973,    0.1764,    0.1669,    0.1722,    0.1923,    0.2245,    0.2639,    0.3052, &
     0.3036,    0.3350,    0.3576,    0.3684,    0.3656,    0.3492,    0.3211,    0.2850, &
     0.2456,    0.2079,    0.1758,    0.1518,    0.1367,    0.1300,    0.1301,    0.1351, &
     0.1428,    0.1512,    0.1581,    0.1621,    0.1621,    0.1577,    0.1490,    0.1362, &
     0.1200,    0.1014,    0.0816,    0.0620,    0.0443,    0.0295,    0.0184,    0.0108, &
     0.0060,    0.0033,    0.0020,    0.0016,    0.0019,    0.0029,    0.0048,    0.0079, &
     0.0124,    0.0188,    0.0276,    0.0396,    0.0554,    0.0761,    0.1026,    0.1354, &
     0.1735,    0.2143,    0.2532,    0.2843,    0.3023,    0.3036,    0.2884,    0.2606, &
     0.2271,    0.1961,    0.1747,    0.1677,    0.1763,    0.1986,    0.2306,    0.2673 &
     /)
!
ZHORCLIM_LAND( 1025 : 1536 ) = (/ &
     0.2577,    0.2888,    0.3130,    0.3264,    0.3263,    0.3122,    0.2857,    0.2504, &
     0.2113,    0.1736,    0.1414,    0.1173,    0.1023,    0.0960,    0.0971,    0.1040, &
     0.1149,    0.1280,    0.1414,    0.1532,    0.1620,    0.1665,    0.1654,    0.1584, &
     0.1453,    0.1269,    0.1048,    0.0814,    0.0592,    0.0402,    0.0255,    0.0151, &
     0.0085,    0.0046,    0.0024,    0.0014,    0.0012,    0.0016,    0.0026,    0.0044, &
     0.0071,    0.0114,    0.0177,    0.0270,    0.0405,    0.0598,    0.0865,    0.1216, &
     0.1644,    0.2123,    0.2597,    0.2998,    0.3255,    0.3323,    0.3191,    0.2894, &
     0.2505,    0.2109,    0.1789,    0.1602,    0.1573,    0.1692,    0.1930,    0.2241, &
     0.2110,    0.2409,    0.2658,    0.2811,    0.2837,    0.2726,    0.2490,    0.2165, &
     0.1798,    0.1440,    0.1133,    0.0901,    0.0756,    0.0696,    0.0713,    0.0795, &
     0.0929,    0.1100,    0.1293,    0.1487,    0.1661,    0.1792,    0.1859,    0.1844, &
     0.1742,    0.1558,    0.1313,    0.1038,    0.0768,    0.0531,    0.0343,    0.0208, &
     0.0120,    0.0065,    0.0034,    0.0018,    0.0011,    0.0011,    0.0015,    0.0025, &
     0.0041,    0.0068,    0.0114,    0.0188,    0.0305,    0.0484,    0.0745,    0.1101, &
     0.1550,    0.2063,    0.2584,    0.3038,    0.3347,    0.3454,    0.3343,    0.3044, &
     0.2625,    0.2174,    0.1777,    0.1499,    0.1372,    0.1399,    0.1557,    0.1809, &
     0.1680,    0.1954,    0.2194,    0.2354,    0.2401,    0.2320,    0.2122,    0.1838, &
     0.1511,    0.1189,    0.0908,    0.0695,    0.0561,    0.0507,    0.0527,    0.0617, &
     0.0768,    0.0970,    0.1209,    0.1464,    0.1710,    0.1915,    0.2047,    0.2081, &
     0.2005,    0.1824,    0.1561,    0.1253,    0.0943,    0.0664,    0.0439,    0.0273, &
     0.0161,    0.0090,    0.0048,    0.0026,    0.0014,    0.0011,    0.0011,    0.0015, &
     0.0025,    0.0043,    0.0078,    0.0140,    0.0243,    0.0408,    0.0655,    0.1000, &
     0.1442,    0.1954,    0.2481,    0.2947,    0.3275,    0.3403,    0.3312,    0.3026, &
     0.2606,    0.2138,    0.1704,    0.1372,    0.1179,    0.1135,    0.1226,    0.1421, &
     0.1310,    0.1547,    0.1764,    0.1916,    0.1972,    0.1918,    0.1760,    0.1524, &
     0.1248,    0.0971,    0.0729,    0.0543,    0.0425,    0.0379,    0.0404,    0.0498, &
     0.0657,    0.0876,    0.1142,    0.1435,    0.1727,    0.1981,    0.2159,    0.2230, &
     0.2177,    0.2005,    0.1738,    0.1416,    0.1083,    0.0778,    0.0526,    0.0336, &
     0.0203,    0.0117,    0.0065,    0.0036,    0.0020,    0.0013,    0.0010,    0.0011, &
     0.0017,    0.0031,    0.0061,    0.0115,    0.0208,    0.0358,    0.0587,    0.0909, &
     0.1322,    0.1801,    0.2297,    0.2738,    0.3052,    0.3183,    0.3108,    0.2846, &
     0.2452,    0.2001,    0.1571,    0.1225,    0.1001,    0.0913,    0.0954,    0.1098, &
     0.1005,    0.1198,    0.1378,    0.1511,    0.1566,    0.1531,    0.1411,    0.1225, &
     0.1003,    0.0779,    0.0580,    0.0427,    0.0331,    0.0296,    0.0325,    0.0419, &
     0.0577,    0.0797,    0.1068,    0.1372,    0.1678,    0.1951,    0.2149,    0.2241, &
     0.2209,    0.2055,    0.1802,    0.1488,    0.1158,    0.0849,    0.0588,    0.0386, &
     0.0242,    0.0145,    0.0084,    0.0048,    0.0027,    0.0017,    0.0012,    0.0011, &
     0.0015,    0.0028,    0.0054,    0.0104,    0.0190,    0.0328,    0.0536,    0.0825, &
     0.1193,    0.1617,    0.2054,    0.2441,    0.2717,    0.2833,    0.2768,    0.2539, &
     0.2189,    0.1785,    0.1391,    0.1065,    0.0841,    0.0734,    0.0740,    0.0840, &
     0.0759,    0.0904,    0.1043,    0.1147,    0.1194,    0.1173,    0.1085,    0.0947, &
     0.0779,    0.0607,    0.0454,    0.0335,    0.0262,    0.0240,    0.0272,    0.0362, &
     0.0511,    0.0716,    0.0970,    0.1256,    0.1547,    0.1807,    0.2001,    0.2099, &
     0.2083,    0.1954,    0.1733,    0.1452,    0.1150,    0.0862,    0.0613,    0.0415, &
     0.0269,    0.0167,    0.0101,    0.0060,    0.0035,    0.0022,    0.0015,    0.0013, &
     0.0016,    0.0029,    0.0055,    0.0103,    0.0184,    0.0310,    0.0494,    0.0745, &
     0.1059,    0.1414,    0.1775,    0.2093,    0.2315,    0.2405,    0.2348,    0.2153, &
     0.1860,    0.1518,    0.1183,    0.0899,    0.0697,    0.0590,    0.0576,    0.0640, &
     0.0561,    0.0661,    0.0759,    0.0833,    0.0868,    0.0855,    0.0795,    0.0697, &
     0.0578,    0.0455,    0.0345,    0.0260,    0.0209,    0.0198,    0.0232,    0.0314, &
     0.0444,    0.0623,    0.0842,    0.1088,    0.1338,    0.1562,    0.1733,    0.1823, &
     0.1818,    0.1720,    0.1542,    0.1311,    0.1058,    0.0812,    0.0593,    0.0415, &
     0.0279,    0.0181,    0.0114,    0.0071,    0.0044,    0.0028,    0.0019,    0.0017, &
     0.0021,    0.0034,    0.0061,    0.0108,    0.0183,    0.0296,    0.0455,    0.0665, &
     0.0920,    0.1202,    0.1483,    0.1725,    0.1890,    0.1951,    0.1898,    0.1741, &
     0.1507,    0.1235,    0.0967,    0.0737,    0.0568,    0.0472,    0.0447,    0.0483, &
     0.0405,    0.0466,    0.0528,    0.0575,    0.0598,    0.0589,    0.0550,    0.0486, &
     0.0408,    0.0326,    0.0253,    0.0197,    0.0166,    0.0164,    0.0197,    0.0266, &
     0.0372,    0.0515,    0.0689,    0.0883,    0.1078,    0.1254,    0.1389,    0.1463, &
     0.1466,    0.1397,    0.1267,    0.1094,    0.0900,    0.0708,    0.0533,    0.0385, &
     0.0269,    0.0182,    0.0120,    0.0078,    0.0050,    0.0033,    0.0025,    0.0022, &
     0.0028,    0.0042,    0.0069,    0.0113,    0.0182,    0.0280,    0.0412,    0.0580, &
     0.0777,    0.0988,    0.1193,    0.1364,    0.1476,    0.1512,    0.1465,    0.1343, &
     0.1167,    0.0963,    0.0761,    0.0586,    0.0454,    0.0374,    0.0345,    0.0359 &
     /)
!
ZHORCLIM_LAND( 1537 : 2048 ) = (/ &
     0.0284,    0.0316,    0.0350,    0.0376,    0.0388,    0.0382,    0.0358,    0.0320, &
     0.0272,    0.0223,    0.0178,    0.0146,    0.0130,    0.0134,    0.0162,    0.0216, &
     0.0297,    0.0402,    0.0528,    0.0667,    0.0806,    0.0931,    0.1027,    0.1082, &
     0.1089,    0.1046,    0.0960,    0.0842,    0.0708,    0.0571,    0.0443,    0.0331, &
     0.0240,    0.0169,    0.0117,    0.0079,    0.0054,    0.0038,    0.0030,    0.0029, &
     0.0035,    0.0050,    0.0076,    0.0117,    0.0176,    0.0257,    0.0362,    0.0488, &
     0.0631,    0.0780,    0.0918,    0.1030,    0.1100,    0.1117,    0.1078,    0.0989, &
     0.0864,    0.0721,    0.0578,    0.0453,    0.0355,    0.0292,    0.0263,    0.0263, &
     0.0194,    0.0206,    0.0221,    0.0233,    0.0237,    0.0233,    0.0219,    0.0198, &
     0.0172,    0.0145,    0.0121,    0.0105,    0.0099,    0.0106,    0.0129,    0.0168, &
     0.0223,    0.0295,    0.0378,    0.0468,    0.0558,    0.0638,    0.0700,    0.0737, &
     0.0744,    0.0721,    0.0670,    0.0598,    0.0514,    0.0425,    0.0340,    0.0264, &
     0.0198,    0.0146,    0.0105,    0.0075,    0.0054,    0.0041,    0.0035,    0.0035, &
     0.0042,    0.0056,    0.0080,    0.0116,    0.0164,    0.0226,    0.0303,    0.0392, &
     0.0489,    0.0585,    0.0672,    0.0740,    0.0779,    0.0784,    0.0754,    0.0694, &
     0.0612,    0.0518,    0.0424,    0.0339,    0.0272,    0.0224,    0.0198,    0.0189, &
     0.0131,    0.0132,    0.0135,    0.0138,    0.0138,    0.0134,    0.0126,    0.0116, &
     0.0103,    0.0090,    0.0080,    0.0074,    0.0074,    0.0081,    0.0098,    0.0123, &
     0.0159,    0.0202,    0.0252,    0.0305,    0.0357,    0.0404,    0.0440,    0.0462, &
     0.0468,    0.0457,    0.0431,    0.0391,    0.0344,    0.0293,    0.0242,    0.0194, &
     0.0152,    0.0116,    0.0088,    0.0066,    0.0051,    0.0042,    0.0038,    0.0039, &
     0.0046,    0.0059,    0.0079,    0.0107,    0.0143,    0.0188,    0.0240,    0.0298, &
     0.0358,    0.0415,    0.0465,    0.0502,    0.0521,    0.0521,    0.0501,    0.0464, &
     0.0414,    0.0357,    0.0299,    0.0246,    0.0202,    0.0169,    0.0147,    0.0135, &
     0.0088,    0.0084,    0.0082,    0.0080,    0.0077,    0.0074,    0.0070,    0.0065, &
     0.0060,    0.0055,    0.0051,    0.0050,    0.0053,    0.0059,    0.0070,    0.0086, &
     0.0106,    0.0130,    0.0157,    0.0185,    0.0213,    0.0237,    0.0256,    0.0268, &
     0.0272,    0.0268,    0.0256,    0.0237,    0.0213,    0.0187,    0.0159,    0.0133, &
     0.0109,    0.0087,    0.0070,    0.0056,    0.0046,    0.0041,    0.0039,    0.0041, &
     0.0047,    0.0058,    0.0073,    0.0093,    0.0117,    0.0146,    0.0178,    0.0212, &
     0.0245,    0.0276,    0.0302,    0.0320,    0.0329,    0.0328,    0.0316,    0.0295, &
     0.0267,    0.0236,    0.0204,    0.0173,    0.0146,    0.0124,    0.0107,    0.0096, &
     0.0061,    0.0055,    0.0051,    0.0048,    0.0045,    0.0042,    0.0040,    0.0037, &
     0.0035,    0.0034,    0.0034,    0.0035,    0.0037,    0.0042,    0.0049,    0.0057, &
     0.0068,    0.0080,    0.0093,    0.0106,    0.0119,    0.0130,    0.0139,    0.0145, &
     0.0147,    0.0146,    0.0141,    0.0133,    0.0123,    0.0111,    0.0099,    0.0086, &
     0.0074,    0.0063,    0.0054,    0.0047,    0.0042,    0.0039,    0.0039,    0.0041, &
     0.0046,    0.0053,    0.0063,    0.0076,    0.0090,    0.0106,    0.0123,    0.0141, &
     0.0157,    0.0172,    0.0184,    0.0192,    0.0195,    0.0194,    0.0188,    0.0178, &
     0.0164,    0.0149,    0.0132,    0.0116,    0.0101,    0.0088,    0.0077,    0.0068, &
     0.0043,    0.0039,    0.0036,    0.0032,    0.0030,    0.0028,    0.0026,    0.0025, &
     0.0024,    0.0024,    0.0025,    0.0026,    0.0028,    0.0030,    0.0034,    0.0038, &
     0.0043,    0.0048,    0.0053,    0.0059,    0.0064,    0.0068,    0.0072,    0.0074, &
     0.0076,    0.0076,    0.0074,    0.0072,    0.0068,    0.0064,    0.0059,    0.0054, &
     0.0050,    0.0045,    0.0042,    0.0039,    0.0037,    0.0037,    0.0037,    0.0039, &
     0.0042,    0.0046,    0.0052,    0.0058,    0.0065,    0.0073,    0.0080,    0.0088, &
     0.0095,    0.0101,    0.0105,    0.0108,    0.0109,    0.0109,    0.0106,    0.0102, &
     0.0096,    0.0089,    0.0082,    0.0075,    0.0067,    0.0060,    0.0054,    0.0048, &
     0.0033,    0.0031,    0.0029,    0.0027,    0.0025,    0.0024,    0.0023,    0.0023, &
     0.0022,    0.0022,    0.0022,    0.0023,    0.0023,    0.0024,    0.0026,    0.0027, &
     0.0029,    0.0031,    0.0032,    0.0034,    0.0036,    0.0037,    0.0038,    0.0039, &
     0.0040,    0.0040,    0.0040,    0.0040,    0.0039,    0.0038,    0.0037,    0.0036, &
     0.0035,    0.0034,    0.0034,    0.0033,    0.0033,    0.0034,    0.0034,    0.0035, &
     0.0037,    0.0039,    0.0041,    0.0043,    0.0045,    0.0048,    0.0050,    0.0053, &
     0.0055,    0.0057,    0.0058,    0.0059,    0.0059,    0.0058,    0.0057,    0.0056, &
     0.0054,    0.0052,    0.0049,    0.0047,    0.0044,    0.0041,    0.0038,    0.0036, &
     0.0028,    0.0028,    0.0027,    0.0027,    0.0026,    0.0026,    0.0026,    0.0025, &
     0.0025,    0.0025,    0.0025,    0.0025,    0.0025,    0.0025,    0.0025,    0.0025, &
     0.0025,    0.0026,    0.0026,    0.0026,    0.0026,    0.0027,    0.0027,    0.0027, &
     0.0027,    0.0028,    0.0028,    0.0028,    0.0028,    0.0028,    0.0028,    0.0029, &
     0.0029,    0.0029,    0.0029,    0.0029,    0.0030,    0.0030,    0.0030,    0.0030, &
     0.0031,    0.0031,    0.0031,    0.0032,    0.0032,    0.0032,    0.0033,    0.0033, &
     0.0033,    0.0033,    0.0033,    0.0033,    0.0033,    0.0033,    0.0033,    0.0033, &
     0.0032,    0.0032,    0.0031,    0.0031,    0.0030,    0.0030,    0.0029,    0.0029 &
     /)
!
! urb
ZHORCLIM_URB( 1 : 512 ) = (/ &
     0.1124,    0.1133,    0.1140,    0.1145,    0.1147,    0.1147,    0.1144,    0.1139, &
     0.1131,    0.1121,    0.1109,    0.1095,    0.1080,    0.1063,    0.1044,    0.1025, &
     0.1005,    0.0984,    0.0963,    0.0942,    0.0921,    0.0900,    0.0880,    0.0861, &
     0.0843,    0.0826,    0.0809,    0.0794,    0.0781,    0.0769,    0.0758,    0.0749, &
     0.0742,    0.0736,    0.0732,    0.0730,    0.0729,    0.0730,    0.0732,    0.0736, &
     0.0742,    0.0749,    0.0758,    0.0768,    0.0780,    0.0793,    0.0807,    0.0822, &
     0.0838,    0.0855,    0.0874,    0.0892,    0.0912,    0.0932,    0.0952,    0.0973, &
     0.0993,    0.1013,    0.1032,    0.1051,    0.1068,    0.1085,    0.1100,    0.1113, &
     0.1686,    0.1728,    0.1760,    0.1782,    0.1792,    0.1791,    0.1779,    0.1756, &
     0.1722,    0.1679,    0.1628,    0.1570,    0.1507,    0.1441,    0.1371,    0.1301, &
     0.1231,    0.1161,    0.1093,    0.1028,    0.0965,    0.0906,    0.0850,    0.0798, &
     0.0750,    0.0705,    0.0665,    0.0628,    0.0596,    0.0568,    0.0544,    0.0525, &
     0.0510,    0.0499,    0.0492,    0.0490,    0.0492,    0.0497,    0.0507,    0.0520, &
     0.0536,    0.0555,    0.0578,    0.0603,    0.0631,    0.0661,    0.0693,    0.0728, &
     0.0766,    0.0807,    0.0850,    0.0896,    0.0946,    0.1000,    0.1057,    0.1117, &
     0.1181,    0.1247,    0.1315,    0.1383,    0.1451,    0.1517,    0.1579,    0.1636, &
     0.2417,    0.2514,    0.2590,    0.2641,    0.2664,    0.2659,    0.2627,    0.2570, &
     0.2489,    0.2389,    0.2274,    0.2147,    0.2013,    0.1877,    0.1741,    0.1608, &
     0.1480,    0.1360,    0.1247,    0.1142,    0.1045,    0.0955,    0.0873,    0.0797, &
     0.0727,    0.0663,    0.0605,    0.0552,    0.0506,    0.0465,    0.0432,    0.0405, &
     0.0385,    0.0372,    0.0367,    0.0368,    0.0377,    0.0392,    0.0414,    0.0441, &
     0.0474,    0.0511,    0.0551,    0.0594,    0.0638,    0.0683,    0.0729,    0.0776, &
     0.0823,    0.0871,    0.0923,    0.0978,    0.1040,    0.1110,    0.1190,    0.1281, &
     0.1383,    0.1498,    0.1623,    0.1756,    0.1895,    0.2036,    0.2173,    0.2301, &
     0.3248,    0.3424,    0.3560,    0.3649,    0.3687,    0.3673,    0.3608,    0.3498, &
     0.3347,    0.3165,    0.2961,    0.2744,    0.2522,    0.2303,    0.2094,    0.1898, &
     0.1718,    0.1556,    0.1411,    0.1281,    0.1164,    0.1058,    0.0960,    0.0869, &
     0.0784,    0.0703,    0.0627,    0.0557,    0.0493,    0.0437,    0.0390,    0.0352, &
     0.0326,    0.0311,    0.0307,    0.0315,    0.0334,    0.0365,    0.0406,    0.0456, &
     0.0515,    0.0581,    0.0651,    0.0723,    0.0795,    0.0864,    0.0928,    0.0985, &
     0.1035,    0.1079,    0.1120,    0.1162,    0.1210,    0.1269,    0.1346,    0.1446, &
     0.1572,    0.1726,    0.1908,    0.2114,    0.2340,    0.2576,    0.2813,    0.3041, &
     0.4053,    0.4321,    0.4527,    0.4659,    0.4710,    0.4680,    0.4570,    0.4391, &
     0.4155,    0.3876,    0.3571,    0.3255,    0.2942,    0.2644,    0.2370,    0.2125, &
     0.1912,    0.1729,    0.1573,    0.1440,    0.1323,    0.1216,    0.1115,    0.1016, &
     0.0916,    0.0817,    0.0719,    0.0624,    0.0536,    0.0457,    0.0390,    0.0338, &
     0.0301,    0.0281,    0.0279,    0.0294,    0.0327,    0.0376,    0.0443,    0.0525, &
     0.0621,    0.0728,    0.0844,    0.0963,    0.1079,    0.1187,    0.1280,    0.1354, &
     0.1407,    0.1438,    0.1451,    0.1455,    0.1458,    0.1473,    0.1514,    0.1591, &
     0.1713,    0.1887,    0.2114,    0.2389,    0.2704,    0.3045,    0.3397,    0.3739, &
     0.4674,    0.5031,    0.5303,    0.5474,    0.5534,    0.5481,    0.5323,    0.5074, &
     0.4752,    0.4380,    0.3982,    0.3580,    0.3194,    0.2838,    0.2524,    0.2257, &
     0.2037,    0.1862,    0.1722,    0.1609,    0.1512,    0.1419,    0.1324,    0.1221, &
     0.1107,    0.0984,    0.0856,    0.0729,    0.0608,    0.0500,    0.0408,    0.0337, &
     0.0288,    0.0263,    0.0261,    0.0284,    0.0331,    0.0401,    0.0496,    0.0614, &
     0.0755,    0.0914,    0.1088,    0.1268,    0.1446,    0.1609,    0.1746,    0.1847, &
     0.1904,    0.1916,    0.1888,    0.1831,    0.1761,    0.1701,    0.1670,    0.1691, &
     0.1779,    0.1945,    0.2193,    0.2519,    0.2909,    0.3345,    0.3803,    0.4256, &
     0.4968,    0.5391,    0.5712,    0.5910,    0.5972,    0.5898,    0.5698,    0.5392, &
     0.5005,    0.4568,    0.4109,    0.3656,    0.3233,    0.2856,    0.2537,    0.2281, &
     0.2087,    0.1946,    0.1846,    0.1773,    0.1710,    0.1642,    0.1558,    0.1450, &
     0.1319,    0.1168,    0.1004,    0.0839,    0.0682,    0.0541,    0.0424,    0.0333, &
     0.0272,    0.0242,    0.0241,    0.0271,    0.0330,    0.0421,    0.0542,    0.0695, &
     0.0880,    0.1093,    0.1330,    0.1579,    0.1827,    0.2057,    0.2248,    0.2382, &
     0.2448,    0.2439,    0.2361,    0.2229,    0.2068,    0.1908,    0.1783,    0.1722, &
     0.1750,    0.1882,    0.2123,    0.2469,    0.2904,    0.3405,    0.3939,    0.4472, &
     0.4863,    0.5314,    0.5655,    0.5860,    0.5918,    0.5828,    0.5605,    0.5272, &
     0.4859,    0.4400,    0.3930,    0.3476,    0.3064,    0.2710,    0.2424,    0.2211, &
     0.2065,    0.1978,    0.1932,    0.1909,    0.1886,    0.1845,    0.1771,    0.1658, &
     0.1506,    0.1323,    0.1123,    0.0920,    0.0728,    0.0559,    0.0420,    0.0316, &
     0.0247,    0.0213,    0.0213,    0.0247,    0.0316,    0.0420,    0.0562,    0.0742, &
     0.0963,    0.1222,    0.1513,    0.1826,    0.2141,    0.2436,    0.2681,    0.2852, &
     0.2927,    0.2898,    0.2769,    0.2561,    0.2306,    0.2044,    0.1819,    0.1669, &
     0.1625,    0.1704,    0.1914,    0.2247,    0.2687,    0.3206,    0.3768,    0.4334 &
     /)
ZHORCLIM_URB( 513 : 1024 ) = (/ &
     0.4380,    0.4816,    0.5142,    0.5336,    0.5385,    0.5291,    0.5069,    0.4745, &
     0.4352,    0.3924,    0.3495,    0.3093,    0.2738,    0.2445,    0.2223,    0.2072, &
     0.1989,    0.1961,    0.1970,    0.1993,    0.2006,    0.1985,    0.1917,    0.1794, &
     0.1620,    0.1408,    0.1176,    0.0944,    0.0728,    0.0542,    0.0392,    0.0282, &
     0.0210,    0.0176,    0.0177,    0.0214,    0.0286,    0.0396,    0.0546,    0.0740, &
     0.0980,    0.1266,    0.1593,    0.1949,    0.2314,    0.2658,    0.2947,    0.3147, &
     0.3230,    0.3184,    0.3014,    0.2742,    0.2410,    0.2065,    0.1757,    0.1531, &
     0.1420,    0.1443,    0.1605,    0.1897,    0.2302,    0.2791,    0.3327,    0.3871, &
     0.3632,    0.4012,    0.4297,    0.4463,    0.4502,    0.4414,    0.4217,    0.3936, &
     0.3603,    0.3249,    0.2905,    0.2592,    0.2328,    0.2121,    0.1977,    0.1896, &
     0.1874,    0.1897,    0.1949,    0.2005,    0.2039,    0.2029,    0.1958,    0.1823, &
     0.1631,    0.1398,    0.1147,    0.0901,    0.0677,    0.0489,    0.0341,    0.0235, &
     0.0168,    0.0137,    0.0139,    0.0174,    0.0243,    0.0349,    0.0495,    0.0686, &
     0.0925,    0.1214,    0.1548,    0.1918,    0.2302,    0.2668,    0.2978,    0.3193, &
     0.3280,    0.3224,    0.3028,    0.2720,    0.2343,    0.1950,    0.1596,    0.1325, &
     0.1168,    0.1144,    0.1253,    0.1488,    0.1828,    0.2246,    0.2711,    0.3186, &
     0.2775,    0.3078,    0.3303,    0.3433,    0.3461,    0.3389,    0.3233,    0.3016, &
     0.2766,    0.2510,    0.2272,    0.2067,    0.1905,    0.1790,    0.1725,    0.1707, &
     0.1732,    0.1791,    0.1867,    0.1937,    0.1976,    0.1962,    0.1882,    0.1735, &
     0.1532,    0.1292,    0.1040,    0.0798,    0.0584,    0.0409,    0.0277,    0.0184, &
     0.0127,    0.0101,    0.0104,    0.0134,    0.0194,    0.0288,    0.0418,    0.0591, &
     0.0809,    0.1076,    0.1389,    0.1739,    0.2108,    0.2463,    0.2768,    0.2979, &
     0.3064,    0.3005,    0.2806,    0.2493,    0.2113,    0.1718,    0.1360,    0.1081, &
     0.0908,    0.0854,    0.0917,    0.1088,    0.1348,    0.1676,    0.2044,    0.2420, &
     0.1963,    0.2182,    0.2345,    0.2437,    0.2455,    0.2403,    0.2292,    0.2144, &
     0.1981,    0.1823,    0.1689,    0.1586,    0.1518,    0.1485,    0.1485,    0.1515, &
     0.1573,    0.1649,    0.1730,    0.1796,    0.1824,    0.1797,    0.1704,    0.1550, &
     0.1346,    0.1114,    0.0878,    0.0658,    0.0470,    0.0320,    0.0210,    0.0136, &
     0.0091,    0.0072,    0.0075,    0.0099,    0.0147,    0.0222,    0.0330,    0.0473, &
     0.0656,    0.0881,    0.1150,    0.1454,    0.1777,    0.2093,    0.2366,    0.2558, &
     0.2635,    0.2581,    0.2399,    0.2115,    0.1770,    0.1414,    0.1091,    0.0838, &
     0.0674,    0.0608,    0.0636,    0.0748,    0.0931,    0.1166,    0.1432,    0.1706, &
     0.1298,    0.1444,    0.1551,    0.1611,    0.1621,    0.1587,    0.1517,    0.1427, &
     0.1336,    0.1260,    0.1208,    0.1185,    0.1189,    0.1216,    0.1262,    0.1324, &
     0.1399,    0.1480,    0.1553,    0.1603,    0.1612,    0.1566,    0.1462,    0.1305, &
     0.1111,    0.0901,    0.0695,    0.0510,    0.0355,    0.0236,    0.0151,    0.0095, &
     0.0063,    0.0050,    0.0052,    0.0070,    0.0105,    0.0161,    0.0241,    0.0350, &
     0.0492,    0.0668,    0.0879,    0.1120,    0.1381,    0.1639,    0.1864,    0.2024, &
     0.2091,    0.2049,    0.1901,    0.1668,    0.1387,    0.1098,    0.0837,    0.0630, &
     0.0491,    0.0426,    0.0429,    0.0495,    0.0611,    0.0766,    0.0943,    0.1126, &
     0.0818,    0.0909,    0.0974,    0.1010,    0.1015,    0.0993,    0.0953,    0.0905, &
     0.0863,    0.0840,    0.0842,    0.0870,    0.0918,    0.0981,    0.1055,    0.1134, &
     0.1216,    0.1294,    0.1355,    0.1386,    0.1374,    0.1312,    0.1200,    0.1048, &
     0.0873,    0.0693,    0.0524,    0.0377,    0.0257,    0.0167,    0.0105,    0.0065, &
     0.0043,    0.0034,    0.0036,    0.0047,    0.0070,    0.0108,    0.0163,    0.0240, &
     0.0340,    0.0467,    0.0620,    0.0797,    0.0990,    0.1184,    0.1356,    0.1482, &
     0.1538,    0.1513,    0.1407,    0.1239,    0.1035,    0.0824,    0.0632,    0.0478, &
     0.0370,    0.0311,    0.0298,    0.0327,    0.0393,    0.0486,    0.0596,    0.0711, &
     0.0508,    0.0562,    0.0601,    0.0622,    0.0624,    0.0610,    0.0586,    0.0562, &
     0.0547,    0.0551,    0.0579,    0.0629,    0.0696,    0.0775,    0.0860,    0.0947, &
     0.1031,    0.1104,    0.1154,    0.1171,    0.1143,    0.1070,    0.0957,    0.0817, &
     0.0666,    0.0519,    0.0387,    0.0275,    0.0186,    0.0120,    0.0074,    0.0045, &
     0.0029,    0.0023,    0.0023,    0.0030,    0.0044,    0.0067,    0.0102,    0.0151, &
     0.0217,    0.0301,    0.0403,    0.0524,    0.0657,    0.0793,    0.0916,    0.1010, &
     0.1059,    0.1052,    0.0991,    0.0888,    0.0759,    0.0623,    0.0497,    0.0391, &
     0.0309,    0.0255,    0.0230,    0.0232,    0.0260,    0.0310,    0.0374,    0.0443, &
     0.0323,    0.0358,    0.0383,    0.0397,    0.0398,    0.0388,    0.0372,    0.0356, &
     0.0351,    0.0362,    0.0395,    0.0448,    0.0516,    0.0595,    0.0680,    0.0767, &
     0.0850,    0.0919,    0.0963,    0.0971,    0.0937,    0.0863,    0.0757,    0.0635, &
     0.0509,    0.0393,    0.0293,    0.0210,    0.0145,    0.0095,    0.0060,    0.0037, &
     0.0023,    0.0017,    0.0016,    0.0019,    0.0026,    0.0038,    0.0058,    0.0087, &
     0.0127,    0.0178,    0.0242,    0.0319,    0.0405,    0.0495,    0.0581,    0.0651, &
     0.0695,    0.0707,    0.0688,    0.0642,    0.0578,    0.0505,    0.0432,    0.0362, &
     0.0299,    0.0247,    0.0209,    0.0190,    0.0191,    0.0210,    0.0243,    0.0283 &
     /)
ZHORCLIM_URB( 1025 : 1536 ) = (/ &
     0.0221,    0.0246,    0.0266,    0.0278,    0.0280,    0.0273,    0.0258,    0.0244, &
     0.0238,    0.0245,    0.0271,    0.0314,    0.0372,    0.0442,    0.0519,    0.0601, &
     0.0679,    0.0745,    0.0786,    0.0793,    0.0762,    0.0696,    0.0605,    0.0504, &
     0.0405,    0.0316,    0.0241,    0.0180,    0.0130,    0.0091,    0.0061,    0.0039, &
     0.0026,    0.0018,    0.0014,    0.0014,    0.0016,    0.0021,    0.0031,    0.0046, &
     0.0068,    0.0097,    0.0134,    0.0181,    0.0234,    0.0293,    0.0353,    0.0407, &
     0.0451,    0.0481,    0.0495,    0.0496,    0.0483,    0.0459,    0.0423,    0.0378, &
     0.0325,    0.0270,    0.0220,    0.0182,    0.0162,    0.0159,    0.0172,    0.0194, &
     0.0166,    0.0188,    0.0208,    0.0221,    0.0225,    0.0219,    0.0205,    0.0189, &
     0.0177,    0.0176,    0.0190,    0.0219,    0.0262,    0.0317,    0.0382,    0.0454, &
     0.0525,    0.0587,    0.0627,    0.0637,    0.0614,    0.0563,    0.0494,    0.0417, &
     0.0344,    0.0280,    0.0225,    0.0179,    0.0138,    0.0104,    0.0075,    0.0053, &
     0.0036,    0.0025,    0.0018,    0.0014,    0.0012,    0.0013,    0.0017,    0.0024, &
     0.0034,    0.0049,    0.0070,    0.0097,    0.0130,    0.0169,    0.0213,    0.0259, &
     0.0306,    0.0351,    0.0393,    0.0428,    0.0453,    0.0462,    0.0452,    0.0421, &
     0.0371,    0.0311,    0.0248,    0.0195,    0.0157,    0.0137,    0.0135,    0.0147, &
     0.0139,    0.0160,    0.0181,    0.0197,    0.0202,    0.0197,    0.0183,    0.0165, &
     0.0148,    0.0138,    0.0140,    0.0154,    0.0181,    0.0220,    0.0272,    0.0331, &
     0.0393,    0.0447,    0.0486,    0.0501,    0.0490,    0.0458,    0.0412,    0.0362, &
     0.0315,    0.0272,    0.0233,    0.0198,    0.0163,    0.0130,    0.0100,    0.0074, &
     0.0053,    0.0037,    0.0025,    0.0017,    0.0013,    0.0011,    0.0011,    0.0013, &
     0.0018,    0.0025,    0.0035,    0.0051,    0.0073,    0.0101,    0.0137,    0.0180, &
     0.0231,    0.0289,    0.0350,    0.0410,    0.0460,    0.0491,    0.0496,    0.0472, &
     0.0422,    0.0355,    0.0282,    0.0215,    0.0163,    0.0131,    0.0119,    0.0123, &
     0.0127,    0.0148,    0.0170,    0.0186,    0.0193,    0.0189,    0.0175,    0.0155, &
     0.0134,    0.0117,    0.0109,    0.0110,    0.0124,    0.0150,    0.0187,    0.0233, &
     0.0283,    0.0330,    0.0365,    0.0384,    0.0386,    0.0373,    0.0352,    0.0327, &
     0.0303,    0.0279,    0.0254,    0.0227,    0.0196,    0.0163,    0.0130,    0.0099, &
     0.0072,    0.0051,    0.0034,    0.0022,    0.0015,    0.0011,    0.0009,    0.0009, &
     0.0010,    0.0013,    0.0019,    0.0030,    0.0046,    0.0069,    0.0101,    0.0145, &
     0.0200,    0.0267,    0.0341,    0.0416,    0.0480,    0.0522,    0.0534,    0.0513, &
     0.0461,    0.0389,    0.0308,    0.0232,    0.0172,    0.0131,    0.0113,    0.0113, &
     0.0120,    0.0140,    0.0161,    0.0177,    0.0184,    0.0181,    0.0167,    0.0147, &
     0.0124,    0.0104,    0.0089,    0.0082,    0.0086,    0.0100,    0.0126,    0.0160, &
     0.0198,    0.0236,    0.0268,    0.0289,    0.0301,    0.0305,    0.0305,    0.0302, &
     0.0297,    0.0289,    0.0276,    0.0255,    0.0227,    0.0193,    0.0158,    0.0123, &
     0.0091,    0.0064,    0.0043,    0.0028,    0.0018,    0.0012,    0.0009,    0.0007, &
     0.0007,    0.0009,    0.0013,    0.0021,    0.0035,    0.0057,    0.0089,    0.0133, &
     0.0192,    0.0263,    0.0343,    0.0423,    0.0492,    0.0537,    0.0551,    0.0530, &
     0.0477,    0.0402,    0.0319,    0.0240,    0.0176,    0.0132,    0.0110,    0.0108, &
     0.0113,    0.0130,    0.0148,    0.0162,    0.0169,    0.0165,    0.0153,    0.0134, &
     0.0112,    0.0091,    0.0073,    0.0063,    0.0060,    0.0067,    0.0084,    0.0108, &
     0.0136,    0.0166,    0.0193,    0.0217,    0.0236,    0.0252,    0.0266,    0.0279, &
     0.0288,    0.0292,    0.0287,    0.0271,    0.0246,    0.0214,    0.0177,    0.0141, &
     0.0106,    0.0077,    0.0053,    0.0035,    0.0022,    0.0014,    0.0009,    0.0007, &
     0.0006,    0.0007,    0.0012,    0.0020,    0.0034,    0.0056,    0.0088,    0.0132, &
     0.0191,    0.0261,    0.0339,    0.0416,    0.0481,    0.0523,    0.0535,    0.0513, &
     0.0462,    0.0390,    0.0310,    0.0234,    0.0172,    0.0129,    0.0107,    0.0103, &
     0.0102,    0.0115,    0.0129,    0.0140,    0.0145,    0.0142,    0.0132,    0.0116, &
     0.0097,    0.0077,    0.0061,    0.0049,    0.0044,    0.0046,    0.0056,    0.0073, &
     0.0094,    0.0117,    0.0140,    0.0163,    0.0185,    0.0208,    0.0231,    0.0252, &
     0.0269,    0.0280,    0.0280,    0.0269,    0.0248,    0.0219,    0.0185,    0.0149, &
     0.0115,    0.0086,    0.0061,    0.0042,    0.0028,    0.0018,    0.0012,    0.0008, &
     0.0007,    0.0008,    0.0013,    0.0022,    0.0036,    0.0058,    0.0090,    0.0132, &
     0.0186,    0.0250,    0.0320,    0.0387,    0.0442,    0.0477,    0.0484,    0.0463, &
     0.0417,    0.0353,    0.0282,    0.0215,    0.0160,    0.0122,    0.0101,    0.0096, &
     0.0089,    0.0097,    0.0106,    0.0113,    0.0116,    0.0114,    0.0106,    0.0094, &
     0.0079,    0.0063,    0.0049,    0.0039,    0.0033,    0.0033,    0.0039,    0.0050, &
     0.0065,    0.0083,    0.0102,    0.0123,    0.0146,    0.0170,    0.0195,    0.0218, &
     0.0238,    0.0251,    0.0254,    0.0247,    0.0230,    0.0206,    0.0177,    0.0146, &
     0.0116,    0.0089,    0.0066,    0.0048,    0.0033,    0.0023,    0.0015,    0.0011, &
     0.0009,    0.0010,    0.0015,    0.0024,    0.0039,    0.0060,    0.0089,    0.0127, &
     0.0173,    0.0226,    0.0282,    0.0335,    0.0377,    0.0403,    0.0406,    0.0387, &
     0.0349,    0.0298,    0.0241,    0.0187,    0.0142,    0.0110,    0.0092,    0.0086 &
     /)
ZHORCLIM_URB( 1537 : 2048 ) = (/ &
     0.0073,    0.0077,    0.0082,    0.0086,    0.0087,    0.0086,    0.0080,    0.0072, &
     0.0061,    0.0050,    0.0040,    0.0032,    0.0027,    0.0025,    0.0028,    0.0035, &
     0.0046,    0.0059,    0.0075,    0.0093,    0.0113,    0.0135,    0.0157,    0.0179, &
     0.0196,    0.0208,    0.0212,    0.0208,    0.0196,    0.0178,    0.0156,    0.0132, &
     0.0108,    0.0085,    0.0066,    0.0050,    0.0036,    0.0026,    0.0019,    0.0014, &
     0.0012,    0.0013,    0.0018,    0.0026,    0.0040,    0.0059,    0.0083,    0.0114, &
     0.0150,    0.0191,    0.0232,    0.0269,    0.0298,    0.0314,    0.0315,    0.0300, &
     0.0272,    0.0234,    0.0193,    0.0153,    0.0120,    0.0095,    0.0080,    0.0074, &
     0.0058,    0.0059,    0.0060,    0.0062,    0.0062,    0.0061,    0.0057,    0.0052, &
     0.0045,    0.0038,    0.0031,    0.0026,    0.0022,    0.0020,    0.0022,    0.0026, &
     0.0033,    0.0042,    0.0054,    0.0068,    0.0085,    0.0102,    0.0120,    0.0136, &
     0.0150,    0.0159,    0.0163,    0.0161,    0.0153,    0.0141,    0.0126,    0.0108, &
     0.0091,    0.0074,    0.0059,    0.0046,    0.0035,    0.0026,    0.0020,    0.0016, &
     0.0014,    0.0015,    0.0020,    0.0027,    0.0039,    0.0054,    0.0073,    0.0096, &
     0.0122,    0.0149,    0.0176,    0.0200,    0.0218,    0.0227,    0.0227,    0.0216, &
     0.0197,    0.0172,    0.0145,    0.0119,    0.0096,    0.0079,    0.0067,    0.0061, &
     0.0044,    0.0043,    0.0042,    0.0042,    0.0042,    0.0041,    0.0038,    0.0035, &
     0.0032,    0.0028,    0.0023,    0.0020,    0.0017,    0.0016,    0.0017,    0.0019, &
     0.0024,    0.0031,    0.0039,    0.0049,    0.0061,    0.0073,    0.0085,    0.0097, &
     0.0106,    0.0112,    0.0115,    0.0114,    0.0110,    0.0102,    0.0092,    0.0081, &
     0.0069,    0.0058,    0.0047,    0.0038,    0.0030,    0.0024,    0.0019,    0.0016, &
     0.0015,    0.0017,    0.0020,    0.0027,    0.0035,    0.0047,    0.0060,    0.0076, &
     0.0092,    0.0109,    0.0125,    0.0139,    0.0148,    0.0153,    0.0152,    0.0145, &
     0.0134,    0.0119,    0.0103,    0.0087,    0.0073,    0.0061,    0.0053,    0.0047, &
     0.0031,    0.0029,    0.0028,    0.0027,    0.0026,    0.0025,    0.0024,    0.0022, &
     0.0020,    0.0018,    0.0016,    0.0014,    0.0013,    0.0013,    0.0013,    0.0015, &
     0.0018,    0.0023,    0.0028,    0.0035,    0.0042,    0.0050,    0.0057,    0.0064, &
     0.0070,    0.0073,    0.0075,    0.0075,    0.0072,    0.0068,    0.0062,    0.0055, &
     0.0048,    0.0041,    0.0035,    0.0028,    0.0023,    0.0019,    0.0017,    0.0015, &
     0.0015,    0.0017,    0.0020,    0.0025,    0.0031,    0.0038,    0.0047,    0.0056, &
     0.0066,    0.0075,    0.0083,    0.0090,    0.0094,    0.0096,    0.0095,    0.0091, &
     0.0085,    0.0077,    0.0069,    0.0060,    0.0051,    0.0044,    0.0039,    0.0034, &
     0.0020,    0.0018,    0.0017,    0.0016,    0.0015,    0.0014,    0.0013,    0.0012, &
     0.0012,    0.0011,    0.0010,    0.0010,    0.0010,    0.0010,    0.0011,    0.0012, &
     0.0015,    0.0017,    0.0021,    0.0025,    0.0029,    0.0033,    0.0037,    0.0040, &
     0.0043,    0.0045,    0.0046,    0.0045,    0.0044,    0.0042,    0.0039,    0.0035, &
     0.0031,    0.0028,    0.0024,    0.0021,    0.0018,    0.0016,    0.0015,    0.0014, &
     0.0015,    0.0016,    0.0018,    0.0021,    0.0025,    0.0030,    0.0034,    0.0039, &
     0.0044,    0.0049,    0.0052,    0.0055,    0.0057,    0.0057,    0.0056,    0.0054, &
     0.0051,    0.0047,    0.0043,    0.0038,    0.0034,    0.0029,    0.0026,    0.0023, &
     0.0013,    0.0011,    0.0010,    0.0009,    0.0009,    0.0008,    0.0008,    0.0007, &
     0.0007,    0.0007,    0.0007,    0.0007,    0.0008,    0.0008,    0.0009,    0.0010, &
     0.0012,    0.0013,    0.0015,    0.0017,    0.0019,    0.0021,    0.0023,    0.0024, &
     0.0026,    0.0026,    0.0027,    0.0026,    0.0026,    0.0025,    0.0024,    0.0022, &
     0.0020,    0.0019,    0.0017,    0.0016,    0.0015,    0.0014,    0.0014,    0.0014, &
     0.0014,    0.0015,    0.0016,    0.0018,    0.0020,    0.0022,    0.0024,    0.0026, &
     0.0028,    0.0030,    0.0031,    0.0032,    0.0033,    0.0032,    0.0032,    0.0031, &
     0.0029,    0.0027,    0.0025,    0.0023,    0.0020,    0.0018,    0.0016,    0.0014, &
     0.0009,    0.0008,    0.0008,    0.0007,    0.0007,    0.0007,    0.0006,    0.0006, &
     0.0006,    0.0006,    0.0007,    0.0007,    0.0007,    0.0008,    0.0008,    0.0009, &
     0.0010,    0.0010,    0.0011,    0.0012,    0.0013,    0.0013,    0.0014,    0.0015, &
     0.0015,    0.0015,    0.0016,    0.0016,    0.0016,    0.0015,    0.0015,    0.0015, &
     0.0014,    0.0014,    0.0013,    0.0013,    0.0013,    0.0013,    0.0013,    0.0013, &
     0.0013,    0.0013,    0.0014,    0.0014,    0.0015,    0.0016,    0.0016,    0.0017, &
     0.0017,    0.0018,    0.0018,    0.0018,    0.0018,    0.0018,    0.0017,    0.0017, &
     0.0016,    0.0015,    0.0015,    0.0014,    0.0013,    0.0012,    0.0011,    0.0010, &
     0.0009,    0.0009,    0.0009,    0.0008,    0.0008,    0.0008,    0.0008,    0.0008, &
     0.0008,    0.0008,    0.0008,    0.0008,    0.0008,    0.0009,    0.0009,    0.0009, &
     0.0009,    0.0009,    0.0009,    0.0009,    0.0010,    0.0010,    0.0010,    0.0010, &
     0.0010,    0.0010,    0.0010,    0.0010,    0.0011,    0.0011,    0.0011,    0.0011, &
     0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011, &
     0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011, &
     0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011,    0.0011, &
     0.0010,    0.0010,    0.0010,    0.0010,    0.0010,    0.0009,    0.0009,    0.0009 &
     /)
!
! desert
ZHORCLIM_DES( 1 : 512 ) = (/ &
     0.0034,    0.0035,    0.0036,    0.0037,    0.0038,    0.0038,    0.0039,    0.0039, &
     0.0040,    0.0040,    0.0040,    0.0039,    0.0039,    0.0039,    0.0038,    0.0038, &
     0.0037,    0.0036,    0.0035,    0.0034,    0.0033,    0.0032,    0.0031,    0.0030, &
     0.0029,    0.0028,    0.0027,    0.0026,    0.0025,    0.0024,    0.0023,    0.0022, &
     0.0021,    0.0021,    0.0020,    0.0020,    0.0019,    0.0019,    0.0018,    0.0018, &
     0.0018,    0.0018,    0.0018,    0.0018,    0.0018,    0.0018,    0.0019,    0.0019, &
     0.0019,    0.0020,    0.0021,    0.0021,    0.0022,    0.0023,    0.0024,    0.0025, &
     0.0026,    0.0027,    0.0028,    0.0029,    0.0030,    0.0031,    0.0032,    0.0033, &
     0.0059,    0.0063,    0.0067,    0.0071,    0.0074,    0.0077,    0.0079,    0.0081, &
     0.0082,    0.0082,    0.0082,    0.0081,    0.0079,    0.0077,    0.0075,    0.0071, &
     0.0068,    0.0064,    0.0060,    0.0056,    0.0051,    0.0047,    0.0043,    0.0039, &
     0.0035,    0.0032,    0.0029,    0.0026,    0.0023,    0.0020,    0.0018,    0.0016, &
     0.0015,    0.0013,    0.0012,    0.0011,    0.0010,    0.0009,    0.0009,    0.0008, &
     0.0008,    0.0008,    0.0008,    0.0008,    0.0008,    0.0008,    0.0009,    0.0009, &
     0.0010,    0.0011,    0.0013,    0.0014,    0.0016,    0.0018,    0.0021,    0.0024, &
     0.0027,    0.0030,    0.0034,    0.0038,    0.0042,    0.0046,    0.0051,    0.0055, &
     0.0106,    0.0115,    0.0124,    0.0132,    0.0139,    0.0145,    0.0150,    0.0154, &
     0.0156,    0.0156,    0.0155,    0.0152,    0.0148,    0.0143,    0.0136,    0.0127, &
     0.0118,    0.0108,    0.0098,    0.0088,    0.0078,    0.0068,    0.0059,    0.0051, &
     0.0044,    0.0037,    0.0031,    0.0026,    0.0022,    0.0018,    0.0015,    0.0013, &
     0.0010,    0.0009,    0.0007,    0.0006,    0.0006,    0.0005,    0.0004,    0.0004, &
     0.0004,    0.0004,    0.0003,    0.0003,    0.0004,    0.0004,    0.0004,    0.0005, &
     0.0006,    0.0007,    0.0009,    0.0011,    0.0014,    0.0018,    0.0022,    0.0028, &
     0.0034,    0.0041,    0.0049,    0.0057,    0.0067,    0.0076,    0.0086,    0.0096, &
     0.0189,    0.0206,    0.0223,    0.0237,    0.0249,    0.0259,    0.0267,    0.0272, &
     0.0275,    0.0274,    0.0271,    0.0265,    0.0255,    0.0243,    0.0228,    0.0211, &
     0.0192,    0.0173,    0.0152,    0.0132,    0.0113,    0.0095,    0.0079,    0.0065, &
     0.0052,    0.0042,    0.0033,    0.0026,    0.0020,    0.0015,    0.0012,    0.0009, &
     0.0007,    0.0005,    0.0004,    0.0003,    0.0003,    0.0002,    0.0002,    0.0002, &
     0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0003, &
     0.0004,    0.0005,    0.0007,    0.0010,    0.0015,    0.0020,    0.0027,    0.0037, &
     0.0048,    0.0061,    0.0076,    0.0093,    0.0111,    0.0130,    0.0150,    0.0170, &
     0.0330,    0.0361,    0.0387,    0.0409,    0.0426,    0.0439,    0.0447,    0.0451, &
     0.0451,    0.0446,    0.0437,    0.0423,    0.0404,    0.0381,    0.0353,    0.0322, &
     0.0289,    0.0255,    0.0220,    0.0186,    0.0155,    0.0126,    0.0100,    0.0079, &
     0.0060,    0.0046,    0.0034,    0.0025,    0.0018,    0.0012,    0.0009,    0.0006, &
     0.0004,    0.0002,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0002,    0.0002, &
     0.0003,    0.0005,    0.0007,    0.0011,    0.0017,    0.0025,    0.0036,    0.0051, &
     0.0070,    0.0093,    0.0121,    0.0152,    0.0187,    0.0223,    0.0260,    0.0296, &
     0.0556,    0.0605,    0.0644,    0.0673,    0.0692,    0.0703,    0.0706,    0.0702, &
     0.0691,    0.0674,    0.0651,    0.0622,    0.0588,    0.0547,    0.0502,    0.0452, &
     0.0400,    0.0346,    0.0293,    0.0243,    0.0197,    0.0155,    0.0120,    0.0090, &
     0.0066,    0.0047,    0.0033,    0.0023,    0.0015,    0.0010,    0.0006,    0.0003, &
     0.0002,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001,    0.0001,    0.0002, &
     0.0003,    0.0005,    0.0008,    0.0012,    0.0020,    0.0031,    0.0047,    0.0070, &
     0.0101,    0.0140,    0.0187,    0.0243,    0.0305,    0.0371,    0.0437,    0.0499, &
     0.0887,    0.0960,    0.1013,    0.1047,    0.1062,    0.1061,    0.1047,    0.1022, &
     0.0988,    0.0948,    0.0900,    0.0846,    0.0787,    0.0723,    0.0654,    0.0582, &
     0.0507,    0.0433,    0.0361,    0.0293,    0.0232,    0.0178,    0.0133,    0.0097, &
     0.0068,    0.0047,    0.0031,    0.0020,    0.0013,    0.0008,    0.0004,    0.0002, &
     0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001,    0.0002, &
     0.0003,    0.0005,    0.0008,    0.0014,    0.0023,    0.0038,    0.0060,    0.0093, &
     0.0139,    0.0200,    0.0278,    0.0370,    0.0473,    0.0583,    0.0693,    0.0796, &
     0.1320,    0.1423,    0.1491,    0.1524,    0.1525,    0.1500,    0.1455,    0.1394, &
     0.1322,    0.1241,    0.1155,    0.1066,    0.0973,    0.0878,    0.0783,    0.0686, &
     0.0591,    0.0497,    0.0408,    0.0326,    0.0252,    0.0190,    0.0138,    0.0097, &
     0.0066,    0.0044,    0.0028,    0.0017,    0.0010,    0.0006,    0.0003,    0.0002, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0002,    0.0002, &
     0.0004,    0.0005,    0.0009,    0.0015,    0.0026,    0.0044,    0.0073,    0.0117, &
     0.0182,    0.0272,    0.0387,    0.0527,    0.0687,    0.0858,    0.1028,    0.1186 &
     /)
ZHORCLIM_DES( 513 : 1024 ) = (/ &
     0.1820,    0.1956,    0.2037,    0.2063,    0.2041,    0.1980,    0.1888,    0.1776, &
     0.1650,    0.1517,    0.1380,    0.1245,    0.1113,    0.0985,    0.0862,    0.0744, &
     0.0631,    0.0524,    0.0424,    0.0333,    0.0254,    0.0186,    0.0132,    0.0091, &
     0.0060,    0.0038,    0.0023,    0.0014,    0.0008,    0.0005,    0.0003,    0.0002, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0002,    0.0003,    0.0004, &
     0.0006,    0.0008,    0.0011,    0.0017,    0.0029,    0.0049,    0.0084,    0.0140, &
     0.0225,    0.0345,    0.0504,    0.0700,    0.0926,    0.1170,    0.1412,    0.1635, &
     0.2317,    0.2485,    0.2575,    0.2589,    0.2536,    0.2429,    0.2283,    0.2110, &
     0.1922,    0.1729,    0.1537,    0.1352,    0.1179,    0.1020,    0.0875,    0.0742, &
     0.0619,    0.0507,    0.0405,    0.0313,    0.0234,    0.0169,    0.0117,    0.0078, &
     0.0050,    0.0031,    0.0018,    0.0010,    0.0006,    0.0003,    0.0002,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0002,    0.0003,    0.0004,    0.0006,    0.0008, &
     0.0010,    0.0012,    0.0015,    0.0021,    0.0032,    0.0053,    0.0092,    0.0157, &
     0.0259,    0.0407,    0.0608,    0.0861,    0.1155,    0.1473,    0.1790,    0.2080, &
     0.2720,    0.2913,    0.3006,    0.3004,    0.2917,    0.2764,    0.2564,    0.2333, &
     0.2088,    0.1838,    0.1596,    0.1368,    0.1162,    0.0979,    0.0819,    0.0680, &
     0.0558,    0.0450,    0.0354,    0.0270,    0.0199,    0.0141,    0.0096,    0.0063, &
     0.0039,    0.0023,    0.0013,    0.0007,    0.0004,    0.0002,    0.0001,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0003,    0.0005,    0.0007,    0.0011,    0.0014, &
     0.0017,    0.0019,    0.0022,    0.0027,    0.0036,    0.0056,    0.0095,    0.0165, &
     0.0278,    0.0447,    0.0681,    0.0978,    0.1328,    0.1710,    0.2091,    0.2438, &
     0.2943,    0.3150,    0.3241,    0.3222,    0.3105,    0.2915,    0.2672,    0.2398, &
     0.2111,    0.1823,    0.1546,    0.1291,    0.1065,    0.0872,    0.0710,    0.0575, &
     0.0463,    0.0367,    0.0284,    0.0214,    0.0155,    0.0108,    0.0072,    0.0046, &
     0.0028,    0.0016,    0.0009,    0.0004,    0.0002,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0001,    0.0002,    0.0004,    0.0007,    0.0011,    0.0016,    0.0021, &
     0.0026,    0.0030,    0.0032,    0.0035,    0.0042,    0.0059,    0.0095,    0.0163, &
     0.0278,    0.0455,    0.0704,    0.1026,    0.1409,    0.1829,    0.2251,    0.2635, &
     0.2939,    0.3145,    0.3228,    0.3194,    0.3058,    0.2846,    0.2582,    0.2289, &
     0.1984,    0.1683,    0.1397,    0.1137,    0.0911,    0.0722,    0.0570,    0.0449, &
     0.0352,    0.0274,    0.0209,    0.0155,    0.0111,    0.0076,    0.0050,    0.0031, &
     0.0019,    0.0010,    0.0005,    0.0003,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0001,    0.0002,    0.0005,    0.0009,    0.0015,    0.0023,    0.0030, &
     0.0037,    0.0042,    0.0044,    0.0045,    0.0048,    0.0060,    0.0090,    0.0151, &
     0.0259,    0.0429,    0.0673,    0.0993,    0.1377,    0.1803,    0.2233,    0.2625, &
     0.2710,    0.2901,    0.2973,    0.2929,    0.2787,    0.2573,    0.2311,    0.2025, &
     0.1733,    0.1446,    0.1177,    0.0935,    0.0728,    0.0558,    0.0425,    0.0324, &
     0.0248,    0.0188,    0.0141,    0.0103,    0.0072,    0.0049,    0.0032,    0.0020, &
     0.0011,    0.0006,    0.0003,    0.0002,    0.0001,    0.0001,    0.0001,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0002,    0.0003,    0.0006,    0.0011,    0.0019,    0.0029,    0.0039, &
     0.0048,    0.0054,    0.0056,    0.0055,    0.0054,    0.0060,    0.0082,    0.0131, &
     0.0224,    0.0375,    0.0594,    0.0886,    0.1242,    0.1639,    0.2044,    0.2414, &
     0.2311,    0.2476,    0.2534,    0.2488,    0.2353,    0.2154,    0.1916,    0.1661, &
     0.1404,    0.1155,    0.0924,    0.0718,    0.0544,    0.0403,    0.0296,    0.0218, &
     0.0161,    0.0119,    0.0087,    0.0063,    0.0043,    0.0029,    0.0019,    0.0011, &
     0.0006,    0.0003,    0.0002,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0002,    0.0004,    0.0007,    0.0013,    0.0022,    0.0034,    0.0046, &
     0.0057,    0.0064,    0.0066,    0.0063,    0.0059,    0.0059,    0.0071,    0.0108, &
     0.0181,    0.0303,    0.0485,    0.0730,    0.1033,    0.1375,    0.1727,    0.2051, &
     0.1825,    0.1960,    0.2004,    0.1960,    0.1842,    0.1672,    0.1473,    0.1263, &
     0.1055,    0.0857,    0.0675,    0.0515,    0.0380,    0.0273,    0.0194,    0.0137, &
     0.0098,    0.0070,    0.0050,    0.0035,    0.0024,    0.0016,    0.0010,    0.0006, &
     0.0003,    0.0002,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0002,    0.0004,    0.0008,    0.0015,    0.0025,    0.0037,    0.0050, &
     0.0062,    0.0069,    0.0071,    0.0067,    0.0060,    0.0055,    0.0060,    0.0084, &
     0.0136,    0.0227,    0.0365,    0.0555,    0.0793,    0.1066,    0.1349,    0.1613 &
     /)
ZHORCLIM_DES( 1025 : 1536 ) = (/ &
     0.1339,    0.1442,    0.1474,    0.1436,    0.1341,    0.1206,    0.1050,    0.0890, &
     0.0735,    0.0590,    0.0459,    0.0345,    0.0249,    0.0174,    0.0119,    0.0081, &
     0.0055,    0.0038,    0.0026,    0.0018,    0.0012,    0.0008,    0.0005,    0.0003, &
     0.0002,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0002,    0.0004,    0.0008,    0.0015,    0.0025,    0.0037,    0.0050, &
     0.0061,    0.0068,    0.0069,    0.0065,    0.0057,    0.0050,    0.0049,    0.0062, &
     0.0096,    0.0158,    0.0255,    0.0391,    0.0564,    0.0765,    0.0977,    0.1177, &
     0.0916,    0.0990,    0.1012,    0.0983,    0.0911,    0.0811,    0.0698,    0.0584, &
     0.0476,    0.0378,    0.0291,    0.0215,    0.0153,    0.0104,    0.0069,    0.0045, &
     0.0030,    0.0020,    0.0013,    0.0009,    0.0006,    0.0004,    0.0002,    0.0001, &
     0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0002,    0.0004,    0.0008,    0.0014,    0.0022,    0.0034,    0.0045, &
     0.0055,    0.0062,    0.0063,    0.0058,    0.0050,    0.0042,    0.0038,    0.0044, &
     0.0063,    0.0103,    0.0166,    0.0256,    0.0372,    0.0511,    0.0659,    0.0800, &
     0.0587,    0.0637,    0.0652,    0.0631,    0.0580,    0.0511,    0.0433,    0.0357, &
     0.0287,    0.0225,    0.0171,    0.0125,    0.0087,    0.0058,    0.0037,    0.0024, &
     0.0015,    0.0009,    0.0006,    0.0004,    0.0002,    0.0002,    0.0001,    0.0001, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0002,    0.0003,    0.0006,    0.0011,    0.0019,    0.0028,    0.0037, &
     0.0046,    0.0051,    0.0051,    0.0048,    0.0041,    0.0033,    0.0028,    0.0029, &
     0.0040,    0.0062,    0.0101,    0.0156,    0.0230,    0.0319,    0.0415,    0.0509, &
     0.0354,    0.0386,    0.0395,    0.0381,    0.0348,    0.0302,    0.0253,    0.0205, &
     0.0162,    0.0125,    0.0094,    0.0068,    0.0046,    0.0030,    0.0019,    0.0011, &
     0.0007,    0.0004,    0.0002,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0001,    0.0002,    0.0005,    0.0009,    0.0014,    0.0021,    0.0028, &
     0.0034,    0.0038,    0.0039,    0.0036,    0.0031,    0.0025,    0.0020,    0.0019, &
     0.0024,    0.0036,    0.0058,    0.0090,    0.0133,    0.0187,    0.0246,    0.0305, &
     0.0201,    0.0220,    0.0225,    0.0216,    0.0196,    0.0169,    0.0139,    0.0111, &
     0.0086,    0.0066,    0.0048,    0.0034,    0.0023,    0.0015,    0.0009,    0.0005, &
     0.0003,    0.0002,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001, &
     0.0001,    0.0001,    0.0002,    0.0003,    0.0006,    0.0010,    0.0015,    0.0019, &
     0.0024,    0.0026,    0.0026,    0.0025,    0.0021,    0.0017,    0.0013,    0.0012, &
     0.0014,    0.0020,    0.0031,    0.0049,    0.0073,    0.0104,    0.0138,    0.0172, &
     0.0107,    0.0117,    0.0119,    0.0115,    0.0103,    0.0088,    0.0072,    0.0057, &
     0.0044,    0.0032,    0.0024,    0.0016,    0.0011,    0.0007,    0.0004,    0.0003, &
     0.0002,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0002,    0.0004,    0.0007,    0.0010,    0.0013, &
     0.0015,    0.0016,    0.0017,    0.0016,    0.0013,    0.0011,    0.0009,    0.0007, &
     0.0008,    0.0010,    0.0016,    0.0025,    0.0038,    0.0054,    0.0073,    0.0091, &
     0.0053,    0.0058,    0.0059,    0.0056,    0.0051,    0.0043,    0.0035,    0.0027, &
     0.0021,    0.0015,    0.0011,    0.0007,    0.0005,    0.0003,    0.0002,    0.0002, &
     0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0002,    0.0003,    0.0004,    0.0006,    0.0008, &
     0.0009,    0.0010,    0.0010,    0.0009,    0.0008,    0.0007,    0.0005,    0.0004, &
     0.0004,    0.0005,    0.0008,    0.0012,    0.0019,    0.0027,    0.0036,    0.0045, &
     0.0025,    0.0027,    0.0027,    0.0026,    0.0023,    0.0020,    0.0016,    0.0013, &
     0.0010,    0.0007,    0.0005,    0.0003,    0.0002,    0.0002,    0.0001,    0.0001, &
     0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0001,    0.0001,    0.0002,    0.0003,    0.0004,    0.0004, &
     0.0005,    0.0005,    0.0005,    0.0005,    0.0004,    0.0004,    0.0003,    0.0003, &
     0.0002,    0.0003,    0.0004,    0.0006,    0.0009,    0.0013,    0.0017,    0.0021 &
     /)
ZHORCLIM_DES( 1537 : 2048 ) = (/ &
     0.0011,    0.0012,    0.0012,    0.0012,    0.0011,    0.0009,    0.0008,    0.0006, &
     0.0004,    0.0003,    0.0002,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0002,    0.0002,    0.0002, &
     0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0002,    0.0002, &
     0.0001,    0.0002,    0.0002,    0.0003,    0.0004,    0.0006,    0.0008,    0.0010, &
     0.0006,    0.0006,    0.0006,    0.0006,    0.0005,    0.0005,    0.0004,    0.0003, &
     0.0002,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0002,    0.0002,    0.0003,    0.0004,    0.0005, &
     0.0003,    0.0003,    0.0003,    0.0003,    0.0003,    0.0002,    0.0002,    0.0001, &
     0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001,    0.0002,    0.0002, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0000,    0.0000,    0.0000,    0.0000,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001, &
     0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001,    0.0001 &
     /)
!
!
END SUBROUTINE INI_CLIMCEP
!
!
!
END SUBROUTINE INI_HOR_AERCLIM
