!MNH_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!--------------- special set of characters for RCS information
!-----------------------------------------------------------------
! $Source$ $Revision$
! MASDEV4_7 modd 2006/06/27 14:19:31
!-----------------------------------------------------------------
!     ###########################
      MODULE MODD_PACK_GR_FIELD_n
!     ###########################
!
!!****  *MODD_PACK_GR_FIELD$n* - declaration of prognostic variables related
!!                          to the ground parameterization
!!
!!    PURPOSE
!!    -------
!       The purpose of this declarative module is to specify  the 
!     prognostic packed variables related to the ground. 
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!      Book2 of documentation of Meso-NH (module MODD_PACK_GR_FIELDn)
!!      Technical Specifications Report of the Meso-NH (chapters 2 and 3)
!!      
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       02/02/99      
!!       Fsolmon       01/06/00 adaptation for patches 
!!       V.Masson      01/05/01 add roughness lengths modified by snow
!!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_TYPE_DATE
!
USE MODD_PARAMETERS, ONLY: JPMODELMAX
IMPLICIT NONE

TYPE PACK_GR_FIELD_t
!
!
!
  INTEGER, DIMENSION(:), POINTER       :: NSIZE_NATURE=>NULL()
! total number of points with natural cover for each patch
!
  INTEGER                            :: NSIZE_TOWN
! total number of points with urbanized cover
!
  INTEGER                            :: NSIZE_SEA
! total number of points with sea
!
  INTEGER                            :: NSIZE_WATER
! total number of points with inland water
!
  INTEGER, DIMENSION(:,:), POINTER :: N1_NATURE=>NULL()
! list of indexes to unpack fields on natural covers for each patch (1st index)
!
  INTEGER, DIMENSION(:,:), POINTER :: N2_NATURE=>NULL()
! list of indexes to unpack fields on natural covers for each patch (2nd index)
!
  INTEGER, DIMENSION(:), POINTER :: N1_TOWN=>NULL()
! list of indexes to unpack fields on urbanized covers (1st index)
!
  INTEGER, DIMENSION(:), POINTER :: N2_TOWN=>NULL()
! list of indexes to unpack fields on urbanized covers (2nd index)
!
  INTEGER, DIMENSION(:), POINTER :: N1_SEA=>NULL()
! list of indexes to unpack fields on sea (1st index)
!
  INTEGER, DIMENSION(:), POINTER :: N2_SEA=>NULL()
! list of indexes to unpack fields on sea (2nd index)
!
  INTEGER, DIMENSION(:), POINTER :: N1_WATER=>NULL()
! list of indexes to unpack fields on inland water (1st index)
!
  INTEGER, DIMENSION(:), POINTER :: N2_WATER=>NULL()
! list of indexes to unpack fields on inland water (2nd index)
!
!*       1.   Physiographic data
!             ------------------
!
!
  REAL, DIMENSION(:), POINTER :: XP_AOSIP=>NULL(),XP_AOSIM=>NULL(),XP_AOSJP=>NULL(),XP_AOSJM=>NULL()
! directional A/S quantities in 4 coordinate directions
! (IP: i index up;  IM: i index down;  JP: j index up;  JM: j index down)
! They are used in soil routines to compute effective roughness length
!
  REAL, DIMENSION(:), POINTER :: XP_HO2IP=>NULL(),XP_HO2IM=>NULL(),XP_HO2JP=>NULL(),XP_HO2JM=>NULL()
! directional h/2 quantities in 4 coordinate directions
! (IP: i index up;  IM: i index down;  JP: j index up;  JM: j index down)
! They are used in soil routines to compute effective roughness length
!
  REAL, DIMENSION(:), POINTER :: XP_Z0REL=>NULL()
! non-directional orographic roughness length
!
  REAL, DIMENSION(:), POINTER :: XP_Z0EFFIM=>NULL(), XP_Z0EFFIP=>NULL(), XP_Z0EFFJM=>NULL(), XP_Z0EFFJP=>NULL()
! non-directional orographic roughness length
!
  REAL, DIMENSION(:), POINTER :: XP_Z0EFF=>NULL()
! effective roughness length
!
  REAL, DIMENSION(:), POINTER :: XP_SSO_STDEV=>NULL()      ! standard deviation of
                                                   ! Subgrid Scale Orography
  REAL, DIMENSION(:), POINTER :: XP_SSO_ANISOTROPY=>NULL() ! anisotropy of S.S.O.
  REAL, DIMENSION(:), POINTER :: XP_SSO_DIRECTION=>NULL()  ! direction of S.S.O.
  REAL, DIMENSION(:), POINTER :: XP_SSO_SLOPE=>NULL()      ! slope of S.S.O.
!
!
  REAL, DIMENSION(:), POINTER      :: XP_Z0_TOWN=>NULL()
! roughness length for momentum in artificial areas
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_TOWN=>NULL()
! Near-InfraRed albedo in artificial areas
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_TOWN=>NULL()
! VISible albedo in artificial areas
!
  REAL, DIMENSION(:), POINTER      :: XP_EMIS_TOWN=>NULL()
! emissivity in artificial areas
!
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_ROOF=>NULL()
! Near-InfraRed albedo of roofs
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_ROOF=>NULL()
! VISible albedo of roofs
!
  REAL, DIMENSION(:), POINTER      :: XP_EMIS_ROOF=>NULL()
! emissivity of roofs
!
  REAL, DIMENSION(:,:), POINTER    :: XP_HC_ROOF=>NULL()
! heat capacity of roof layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_TC_ROOF=>NULL()
! thermal conductivity of roof layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_D_ROOF=>NULL()
! depth of roof layers
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_ROAD=>NULL()
! Near-InfraRed albedo of roads
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_ROAD=>NULL()
! VISible albedo of roads
!
  REAL, DIMENSION(:), POINTER      :: XP_EMIS_ROAD=>NULL()
! emissivity of roads
!
  REAL, DIMENSION(:,:), POINTER    :: XP_HC_ROAD=>NULL()
! heat capacity of road layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_TC_ROAD=>NULL()
! thermal conductivity of road layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_D_ROAD=>NULL()
! depth of road layers
!
  REAL, DIMENSION(:), POINTER      :: XP_SVF_ROAD=>NULL()
! sky view factor of roads
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_WALL=>NULL()
! Near-InfraRed albedo of walls
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_WALL=>NULL()
! VISible albedo of walls
!
  REAL, DIMENSION(:), POINTER      :: XP_EMIS_WALL=>NULL()
! emissivity of walls
!
  REAL, DIMENSION(:,:), POINTER    :: XP_HC_WALL=>NULL()
! heat capacity of wall layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_TC_WALL=>NULL()
! thermal conductivity of wall layers
!
  REAL, DIMENSION(:,:), POINTER    :: XP_D_WALL=>NULL()
! depth of wall layers
!
  REAL, DIMENSION(:), POINTER      :: XP_SVF_WALL=>NULL()
! sky view factor of walls
!
  REAL, DIMENSION(:), POINTER      :: XP_BLD_HEIGHT=>NULL()
! building height h
!
  REAL, DIMENSION(:), POINTER      :: XP_BLD_HL_RATIO=>NULL()
! building h/L aspect ratio
!
  REAL, DIMENSION(:), POINTER      :: XP_BLD=>NULL()
! fraction of buildings in artificial areas (only)
!
  REAL, DIMENSION(:), POINTER      :: XP_CAN_HW_RATIO=>NULL()
! canyon h/W aspect ratio
!
  REAL, DIMENSION(:), POINTER :: XP_CLAY=>NULL()
! clay fraction in the soil
!
  REAL, DIMENSION(:), POINTER :: XP_SAND=>NULL()
! sand fraction in the soil
!
  REAL, DIMENSION(:), POINTER :: XP_RSMIN=>NULL()
! minimum stomatal resistance
!
  REAL, DIMENSION(:), POINTER :: XP_GAMMA=>NULL()
! parameter in the calculation of Rs
!
  REAL, DIMENSION(:), POINTER :: XP_RGL=>NULL()
! parameter in the calculation of Rs
!
  REAL, DIMENSION(:), POINTER :: XP_CV=>NULL()
! inverse of the heat capacity of the vegetation
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_VEG=>NULL()
! Near-InfraRed albedo in natural areas
!
  REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_VEG=>NULL()
! VISible albedo in natural areas
!
  REAL, DIMENSION(:), POINTER      :: XP_EMIS_ECO=>NULL()
! emissivity of natural areas
!
  REAL, DIMENSION(:,:), POINTER    :: XP_D_G=>NULL()
! soil layer depths
!
  REAL, DIMENSION(:), POINTER      :: XP_C4B=>NULL()
! fiiting soil paramater for vertical diffusion (C4)
!
  REAL, DIMENSION(:), POINTER      :: XP_C4REF=>NULL()
! fiiting soil paramater for vertical diffusion (C4)
!
  REAL, DIMENSION(:), POINTER      :: XP_RUNOFFB=>NULL()
! runoff parameter
!
  REAL, DIMENSION(:), POINTER      :: XP_TAUICE=>NULL()
! characteristic time scale for phase change within the soil
!
  REAL, DIMENSION(:), POINTER      :: XP_CO2FLUX=>NULL()
! CO2 flux towards atmosphere
!
  REAL, DIMENSION(:,:), POINTER    :: XP_VEGTYPE=>NULL()
! fraction of each vegetation type
!
  REAL,  DIMENSION(:), POINTER     :: XP_BSLAI=>NULL()
! ratio d(biomass)/d(lai)
!
  REAL,  DIMENSION(:), POINTER     :: XP_LAIMIN=>NULL()
! minimum LAI
!
  REAL,  DIMENSION(:), POINTER     :: XP_SEFOLD=>NULL()
! e-folding time for senescence (days)
!
  REAL,  DIMENSION(:), POINTER     :: XP_H_TREE=>NULL()
! height of trees
!
  REAL,  DIMENSION(:), POINTER     :: XP_CSP=>NULL()
! atmospheric CO2 concentration
!
  REAL,  DIMENSION(:), POINTER     :: XP_ANMAX=>NULL()
! maximum photosynthesis rate
!
  REAL,  DIMENSION(:), POINTER     :: XP_GMES=>NULL()
! mesophyll conductance (mm s-1)
!
  REAL,  DIMENSION(:), POINTER :: XP_FZERO=>NULL()
! ideal value of F, no photo-respiration or saturation deficit
!
  REAL,  DIMENSION(:), POINTER :: XP_EPSO=>NULL()
! maximum initial quantum use efficiency (mg J-1 PAR)
!
  REAL,  DIMENSION(:), POINTER :: XP_GAMM=>NULL()
! CO2 conpensation concentration (ppm)
!
  REAL,  DIMENSION(:), POINTER :: XP_QDGAMM=>NULL()
! Q10 function for CO2 conpensation concentration
!
  REAL,  DIMENSION(:), POINTER :: XP_QDGMES=>NULL()
! Q10 function for mesophyll conductance 
!
  REAL,  DIMENSION(:), POINTER :: XP_T1GMES=>NULL()
! reference temperature for computing compensation concentration function for 
! mesophyll conductance: minimum temperature 
!
  REAL,  DIMENSION(:), POINTER :: XP_T2GMES=>NULL()
! reference temperature for computing compensation concentration function for 
! mesophyll conductance: maximum temperature
!
  REAL,  DIMENSION(:), POINTER :: XP_AMAX=>NULL()
! leaf photosynthetic capacity (mg m-2 s-1)
!
  REAL,  DIMENSION(:), POINTER :: XP_QDAMAX=>NULL()
! Q10 function for leaf photosynthetic capacity
!
  REAL,  DIMENSION(:), POINTER :: XP_T1AMAX=>NULL()
! reference temperature for computing compensation concentration function for leaf 
! photosynthetic capacity: minimum temperature
!
  REAL,  DIMENSION(:), POINTER :: XP_T2AMAX=>NULL()
! reference temperature for computing compensation concentration function for leaf 
! photosynthetic capacity: maximum temperature
!
  REAL, DIMENSION(:), POINTER :: XP_WSAT=>NULL(), XP_WWILT=>NULL(),  &
        XP_WFC=>NULL(), XP_BCOEF=>NULL(), XP_CGSAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_C1SAT=>NULL(), XP_C2REF=>NULL(),  &
        XP_C3=>NULL(), XP_ACOEF=>NULL(), XP_PCOEF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_HCAPSOIL=>NULL(),  &
        XP_CONDDRY=>NULL(), XP_CONDSLD=>NULL()
!
  REAL, DIMENSION(:), POINTER :: XP_TDEEP=>NULL(), XP_GAMMAT=>NULL()
!
!*       2.   Prognostic and semi-prognostic variables
!             ----------------------------------------
!
!
  REAL, DIMENSION(:), POINTER :: XP_TS_TOWN=>NULL()
! town surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_ALB_TOWN=>NULL()
! town albedo
!
  REAL, DIMENSION(:), POINTER :: XP_T_CANYON=>NULL()
! canyon temperature
!
  REAL, DIMENSION(:), POINTER :: XP_R_CANYON=>NULL()
! canyon vapor mixing ratio
!
  REAL, DIMENSION(:), POINTER :: XP_Q_CANYON=>NULL()
! canyon specific humidity
!
  REAL, DIMENSION(:), POINTER :: XP_U_CANYON=>NULL()
! canyon horizontal wind
!
  REAL, DIMENSION(:), POINTER :: XP_TS_ROOF=>NULL()
! roof surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_TS_ROAD=>NULL()
! road surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_TS_WALL=>NULL()
! wall surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_TI_ROAD=>NULL()
! deep road temperature
!
  REAL, DIMENSION(:), POINTER :: XP_TI_BLD=>NULL()
! building interior temperature
!
  REAL, DIMENSION(:,:), POINTER :: XP_T_ROOF=>NULL()
! roof layers temperature
!
  REAL, DIMENSION(:,:), POINTER :: XP_T_ROAD=>NULL()
! road layers temperatures
!
  REAL, DIMENSION(:,:), POINTER :: XP_T_WALL=>NULL()
! wall layers temperatures
!
  REAL, DIMENSION(:), POINTER :: XP_WS_ROOF=>NULL()
! roof surface reservoir
!
  REAL, DIMENSION(:), POINTER :: XP_WS_ROAD=>NULL()
! road surface reservoir
!
  REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_ROOF=>NULL()
! roof snow layer reservoirs
!
  REAL, DIMENSION(:,:), POINTER :: XP_TSNOW_ROOF=>NULL()
! roof snow layer temperatures
!
  REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_ROOF=>NULL()
! roof snow layer densities
!
  REAL, DIMENSION(:), POINTER :: XP_ASNOW_ROOF=>NULL()
! roof snow albedo
!
  REAL, DIMENSION(:), POINTER :: XP_TSSNOW_ROOF=>NULL()
! roof snow surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_ESNOW_ROOF=>NULL()
! roof snow emissivity
!
  REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_ROAD=>NULL()
! road snow layer reservoirs
!
  REAL, DIMENSION(:,:), POINTER :: XP_TSNOW_ROAD=>NULL()
! road snow layer temperatures
!
  REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_ROAD=>NULL()
! road snow layer densities
!
  REAL, DIMENSION(:), POINTER :: XP_ASNOW_ROAD=>NULL()
! road snow albedo
!
  REAL, DIMENSION(:), POINTER :: XP_TSSNOW_ROAD=>NULL()
! road snow surface temperature
!
  REAL, DIMENSION(:), POINTER :: XP_ESNOW_ROAD=>NULL()
! road snow emissivity
!
  REAL, DIMENSION(:), POINTER :: XP_Z0NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_Z0HNAT=>NULL()
!
!*       3.   Surface fluxes
!             --------------
!
  REAL, DIMENSION(:), POINTER :: XP_ALB_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ALB_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ALB_WALL=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RN_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GFLUX_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RN_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GFLUX_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RN_WALL=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_WALL=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_WALL=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GFLUX_WALL=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RNSNOW_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_HSNOW_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LESNOW_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GSNOW_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_MELT_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RNSNOW_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_HSNOW_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LESNOW_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GSNOW_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_MELT_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RN_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GFLUX_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RUNOFF_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFU_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFV_TOWN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LES_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LES_ROAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RUNOFF_ROOF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RUNOFF_ROAD=>NULL()
!
!*       4.   forcing variables
!             -----------------
!
  REAL, DIMENSION(:), POINTER :: XP_PS=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_PA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_EXNS=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_EXNA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_TA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_QA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RHOA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LW_RAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_DIR_SW_RAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SCA_SW_RAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_TANZEN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RAIN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SNOW=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ZREF=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_DIRCOSZW=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_USLOPE=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_VSLOPE=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_VMOD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_TRAFFIC=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_TRAFFIC=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_INDUSTRY=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_INDUSTRY=>NULL()
!
!*       5.   prognostic or semi-prognostic variables
!             ---------------------------------------
!
  LOGICAL, DIMENSION(:), POINTER :: LP_MOW=>NULL()
  REAL, DIMENSION(:,:), POINTER :: XP_WG=>NULL()
  REAL, DIMENSION(:,:), POINTER :: XP_TG=>NULL()
  REAL, DIMENSION(:,:), POINTER :: XP_WGI=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RS=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_TS=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WR=>NULL()
  REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_VEG=>NULL()
  REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_VEG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ASNOW_VEG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_PSN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_PSNV=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_PSNG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_VEG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LAI=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_Z0VEG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_Z0HVEG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ANFM=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ANF=>NULL()
!
  REAL, DIMENSION(:), POINTER :: XP_WG1=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_T2=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WG2=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WG3=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WGI1=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WGI2=>NULL()
!
!*       6.   forcing and input variables
!             ---------------------------
!
  REAL, DIMENSION(:), POINTER :: XP_SUNTIME=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SW_RAD=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ALFA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_ALB_ECO=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_THA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_D2=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_D3=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WDRAIN=>NULL()
!
!*       7.   fluxes and other outputs
!             ------------------------
!
  REAL, DIMENSION(:), POINTER :: XP_RESA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CH_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CD_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CDN_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RI_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_HU_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_HUG_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RN_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LEG_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LEV_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LES_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LER_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LETR_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_GFLUX_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFU_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFV_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_DRAIN_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RUNOFF_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_MELT_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CO2FLUX_NAT=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_AN=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CG=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_C1=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_C2=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_WGEQ=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CT=>NULL()
!
!*       8.   fields on sea
!             -------------
!
  REAL, DIMENSION(:), POINTER :: XP_Z0SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SST=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFU_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFV_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CD_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CDN_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CH_SEA=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RI_SEA=>NULL()
!
!*       9.   fields on inland water
!             ----------------------
!
  REAL, DIMENSION(:), POINTER :: XP_Z0WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_TS_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_H_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_LE_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFU_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_SFV_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CD_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CDN_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_CH_WATER=>NULL()
  REAL, DIMENSION(:), POINTER :: XP_RI_WATER=>NULL()
!
!*      10.   fields on town
!             --------------
!
  REAL, DIMENSION(:), POINTER :: XP_CH_TOWN=>NULL()
!
!
END TYPE PACK_GR_FIELD_t

TYPE(PACK_GR_FIELD_t), DIMENSION(JPMODELMAX), TARGET, SAVE :: PACK_GR_FIELD_MODEL

INTEGER, DIMENSION(:), POINTER       :: NSIZE_NATURE=>NULL()
INTEGER, POINTER :: NSIZE_TOWN=>NULL()
INTEGER, POINTER :: NSIZE_SEA=>NULL()
INTEGER, POINTER :: NSIZE_WATER=>NULL()
INTEGER, DIMENSION(:,:), POINTER :: N1_NATURE=>NULL()
INTEGER, DIMENSION(:,:), POINTER :: N2_NATURE=>NULL()
INTEGER, DIMENSION(:), POINTER :: N1_TOWN=>NULL()
INTEGER, DIMENSION(:), POINTER :: N2_TOWN=>NULL()
INTEGER, DIMENSION(:), POINTER :: N1_SEA=>NULL()
INTEGER, DIMENSION(:), POINTER :: N2_SEA=>NULL()
INTEGER, DIMENSION(:), POINTER :: N1_WATER=>NULL()
INTEGER, DIMENSION(:), POINTER :: N2_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_AOSIP=>NULL(),XP_AOSIM=>NULL(), &
      XP_AOSJP=>NULL(),XP_AOSJM=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HO2IP=>NULL(),XP_HO2IM=>NULL(), &
      XP_HO2JP=>NULL(),XP_HO2JM=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0REL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0EFFIM=>NULL(), XP_Z0EFFIP=>NULL(), &
      XP_Z0EFFJM=>NULL(), XP_Z0EFFJP=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0EFF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SSO_STDEV=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SSO_ANISOTROPY=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SSO_DIRECTION=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SSO_SLOPE=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_Z0_TOWN=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_TOWN=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_TOWN=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_EMIS_TOWN=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_ROOF=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_ROOF=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_EMIS_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_HC_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_TC_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_D_ROOF=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_ROAD=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_ROAD=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_EMIS_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_HC_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_TC_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_D_ROAD=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_SVF_ROAD=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_WALL=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_WALL=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_EMIS_WALL=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_HC_WALL=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_TC_WALL=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_D_WALL=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_SVF_WALL=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_BLD_HEIGHT=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_BLD_HL_RATIO=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_BLD=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_CAN_HW_RATIO=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CLAY=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SAND=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RSMIN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GAMMA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RGL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CV=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBNIR_VEG=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_ALBVIS_VEG=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_EMIS_ECO=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_D_G=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_C4B=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_C4REF=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_RUNOFFB=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_TAUICE=>NULL()
REAL, DIMENSION(:), POINTER      :: XP_CO2FLUX=>NULL()
REAL, DIMENSION(:,:), POINTER    :: XP_VEGTYPE=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_BSLAI=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_LAIMIN=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_SEFOLD=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_H_TREE=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_CSP=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_ANMAX=>NULL()
REAL,  DIMENSION(:), POINTER     :: XP_GMES=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_FZERO=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_EPSO=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_GAMM=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_QDGAMM=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_QDGMES=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_T1GMES=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_T2GMES=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_AMAX=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_QDAMAX=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_T1AMAX=>NULL()
REAL,  DIMENSION(:), POINTER :: XP_T2AMAX=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WSAT=>NULL(), XP_WWILT=>NULL(), &
      XP_WFC=>NULL(), XP_BCOEF=>NULL(), XP_CGSAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_C1SAT=>NULL(), XP_C2REF=>NULL(), &
      XP_C3=>NULL(), XP_ACOEF=>NULL(), XP_PCOEF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HCAPSOIL=>NULL(),XP_CONDDRY=>NULL(),XP_CONDSLD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TDEEP=>NULL(), XP_GAMMAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALB_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_T_CANYON=>NULL()
REAL, DIMENSION(:), POINTER :: XP_R_CANYON=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Q_CANYON=>NULL()
REAL, DIMENSION(:), POINTER :: XP_U_CANYON=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TI_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TI_BLD=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_T_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_T_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_T_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WS_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WS_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_TSNOW_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ASNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TSSNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ESNOW_ROOF=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_TSNOW_ROAD=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ASNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TSSNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ESNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0HNAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALB_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALB_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALB_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RN_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GFLUX_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RN_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GFLUX_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RN_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GFLUX_WALL=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RNSNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HSNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LESNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GSNOW_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_MELT_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RNSNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HSNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LESNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GSNOW_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_MELT_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RN_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GFLUX_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RUNOFF_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFU_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFV_TOWN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LES_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LES_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RUNOFF_ROOF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RUNOFF_ROAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_PS=>NULL()
REAL, DIMENSION(:), POINTER :: XP_PA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_EXNS=>NULL()
REAL, DIMENSION(:), POINTER :: XP_EXNA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_QA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RHOA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LW_RAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_DIR_SW_RAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SCA_SW_RAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TANZEN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RAIN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SNOW=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ZREF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_DIRCOSZW=>NULL()
REAL, DIMENSION(:), POINTER :: XP_USLOPE=>NULL()
REAL, DIMENSION(:), POINTER :: XP_VSLOPE=>NULL()
REAL, DIMENSION(:), POINTER :: XP_VMOD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_TRAFFIC=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_TRAFFIC=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_INDUSTRY=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_INDUSTRY=>NULL()
LOGICAL, DIMENSION(:), POINTER :: LP_MOW=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_WG=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_TG=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_WGI=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RS=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WR=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_WSNOW_VEG=>NULL()
REAL, DIMENSION(:,:), POINTER :: XP_RSNOW_VEG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ASNOW_VEG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_PSN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_PSNV=>NULL()
REAL, DIMENSION(:), POINTER :: XP_PSNG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_VEG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LAI=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0VEG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0HVEG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ANFM=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ANF=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WG1=>NULL()
REAL, DIMENSION(:), POINTER :: XP_T2=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WG2=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WG3=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WGI1=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WGI2=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SUNTIME=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SW_RAD=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALFA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_ALB_ECO=>NULL()
REAL, DIMENSION(:), POINTER :: XP_THA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_D2=>NULL()
REAL, DIMENSION(:), POINTER :: XP_D3=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WDRAIN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RESA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CH_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CD_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CDN_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RI_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HU_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_HUG_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RN_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LEG_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LEV_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LES_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LER_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LETR_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_GFLUX_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFU_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFV_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_DRAIN_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RUNOFF_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_MELT_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CO2FLUX_NAT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_AN=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CG=>NULL()
REAL, DIMENSION(:), POINTER :: XP_C1=>NULL()
REAL, DIMENSION(:), POINTER :: XP_C2=>NULL()
REAL, DIMENSION(:), POINTER :: XP_WGEQ=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CT=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SST=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFU_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFV_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CD_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CDN_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CH_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RI_SEA=>NULL()
REAL, DIMENSION(:), POINTER :: XP_Z0WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_TS_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_H_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_LE_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFU_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_SFV_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CD_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CDN_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CH_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_RI_WATER=>NULL()
REAL, DIMENSION(:), POINTER :: XP_CH_TOWN=>NULL()

CONTAINS

SUBROUTINE PACK_GR_FIELD_GOTO_MODEL(KFROM, KTO)
INTEGER, INTENT(IN) :: KFROM, KTO
!
! Save current state for allocated arrays
PACK_GR_FIELD_MODEL(KFROM)%NSIZE_NATURE=>NSIZE_NATURE
PACK_GR_FIELD_MODEL(KFROM)%N1_NATURE=>N1_NATURE
PACK_GR_FIELD_MODEL(KFROM)%N2_NATURE=>N2_NATURE
PACK_GR_FIELD_MODEL(KFROM)%N1_TOWN=>N1_TOWN
PACK_GR_FIELD_MODEL(KFROM)%N2_TOWN=>N2_TOWN
PACK_GR_FIELD_MODEL(KFROM)%N1_SEA=>N1_SEA
PACK_GR_FIELD_MODEL(KFROM)%N2_SEA=>N2_SEA
PACK_GR_FIELD_MODEL(KFROM)%N1_WATER=>N1_WATER
PACK_GR_FIELD_MODEL(KFROM)%N2_WATER=>N2_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_AOSIP=>XP_AOSIP
PACK_GR_FIELD_MODEL(KFROM)%XP_AOSIM=>XP_AOSIM
PACK_GR_FIELD_MODEL(KFROM)%XP_AOSJP=>XP_AOSJP
PACK_GR_FIELD_MODEL(KFROM)%XP_AOSJM=>XP_AOSJM
PACK_GR_FIELD_MODEL(KFROM)%XP_HO2IP=>XP_HO2IP
PACK_GR_FIELD_MODEL(KFROM)%XP_HO2IM=>XP_HO2IM
PACK_GR_FIELD_MODEL(KFROM)%XP_HO2JP=>XP_HO2JP
PACK_GR_FIELD_MODEL(KFROM)%XP_HO2JM=>XP_HO2JM
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0REL=>XP_Z0REL
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0EFFIM=>XP_Z0EFFIM
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0EFFIP=>XP_Z0EFFIP
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0EFFJM=>XP_Z0EFFJM
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0EFFJP=>XP_Z0EFFJP
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0EFF=>XP_Z0EFF
PACK_GR_FIELD_MODEL(KFROM)%XP_SSO_STDEV=>XP_SSO_STDEV
PACK_GR_FIELD_MODEL(KFROM)%XP_SSO_ANISOTROPY=>XP_SSO_ANISOTROPY
PACK_GR_FIELD_MODEL(KFROM)%XP_SSO_DIRECTION=>XP_SSO_DIRECTION
PACK_GR_FIELD_MODEL(KFROM)%XP_SSO_SLOPE=>XP_SSO_SLOPE
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0_TOWN=>XP_Z0_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBNIR_TOWN=>XP_ALBNIR_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBVIS_TOWN=>XP_ALBVIS_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_EMIS_TOWN=>XP_EMIS_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBNIR_ROOF=>XP_ALBNIR_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBVIS_ROOF=>XP_ALBVIS_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_EMIS_ROOF=>XP_EMIS_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_HC_ROOF=>XP_HC_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_TC_ROOF=>XP_TC_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_D_ROOF=>XP_D_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBNIR_ROAD=>XP_ALBNIR_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBVIS_ROAD=>XP_ALBVIS_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_EMIS_ROAD=>XP_EMIS_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_HC_ROAD=>XP_HC_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TC_ROAD=>XP_TC_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_D_ROAD=>XP_D_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_SVF_ROAD=>XP_SVF_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBNIR_WALL=>XP_ALBNIR_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBVIS_WALL=>XP_ALBVIS_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_EMIS_WALL=>XP_EMIS_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_HC_WALL=>XP_HC_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_TC_WALL=>XP_TC_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_D_WALL=>XP_D_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_SVF_WALL=>XP_SVF_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_BLD_HEIGHT=>XP_BLD_HEIGHT
PACK_GR_FIELD_MODEL(KFROM)%XP_BLD_HL_RATIO=>XP_BLD_HL_RATIO
PACK_GR_FIELD_MODEL(KFROM)%XP_BLD=>XP_BLD
PACK_GR_FIELD_MODEL(KFROM)%XP_CAN_HW_RATIO=>XP_CAN_HW_RATIO
PACK_GR_FIELD_MODEL(KFROM)%XP_CLAY=>XP_CLAY
PACK_GR_FIELD_MODEL(KFROM)%XP_SAND=>XP_SAND
PACK_GR_FIELD_MODEL(KFROM)%XP_RSMIN=>XP_RSMIN
PACK_GR_FIELD_MODEL(KFROM)%XP_GAMMA=>XP_GAMMA
PACK_GR_FIELD_MODEL(KFROM)%XP_RGL=>XP_RGL
PACK_GR_FIELD_MODEL(KFROM)%XP_CV=>XP_CV
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBNIR_VEG=>XP_ALBNIR_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_ALBVIS_VEG=>XP_ALBVIS_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_EMIS_ECO=>XP_EMIS_ECO
PACK_GR_FIELD_MODEL(KFROM)%XP_D_G=>XP_D_G
PACK_GR_FIELD_MODEL(KFROM)%XP_C4B=>XP_C4B
PACK_GR_FIELD_MODEL(KFROM)%XP_C4REF=>XP_C4REF
PACK_GR_FIELD_MODEL(KFROM)%XP_RUNOFFB=>XP_RUNOFFB
PACK_GR_FIELD_MODEL(KFROM)%XP_TAUICE=>XP_TAUICE
PACK_GR_FIELD_MODEL(KFROM)%XP_CO2FLUX=>XP_CO2FLUX
PACK_GR_FIELD_MODEL(KFROM)%XP_VEGTYPE=>XP_VEGTYPE
PACK_GR_FIELD_MODEL(KFROM)%XP_BSLAI=>XP_BSLAI
PACK_GR_FIELD_MODEL(KFROM)%XP_LAIMIN=>XP_LAIMIN
PACK_GR_FIELD_MODEL(KFROM)%XP_SEFOLD=>XP_SEFOLD
PACK_GR_FIELD_MODEL(KFROM)%XP_H_TREE=>XP_H_TREE
PACK_GR_FIELD_MODEL(KFROM)%XP_CSP=>XP_CSP
PACK_GR_FIELD_MODEL(KFROM)%XP_ANMAX=>XP_ANMAX
PACK_GR_FIELD_MODEL(KFROM)%XP_GMES=>XP_GMES
PACK_GR_FIELD_MODEL(KFROM)%XP_FZERO=>XP_FZERO
PACK_GR_FIELD_MODEL(KFROM)%XP_EPSO=>XP_EPSO
PACK_GR_FIELD_MODEL(KFROM)%XP_GAMM=>XP_GAMM
PACK_GR_FIELD_MODEL(KFROM)%XP_QDGAMM=>XP_QDGAMM
PACK_GR_FIELD_MODEL(KFROM)%XP_QDGMES=>XP_QDGMES
PACK_GR_FIELD_MODEL(KFROM)%XP_T1GMES=>XP_T1GMES
PACK_GR_FIELD_MODEL(KFROM)%XP_T2GMES=>XP_T2GMES
PACK_GR_FIELD_MODEL(KFROM)%XP_AMAX=>XP_AMAX
PACK_GR_FIELD_MODEL(KFROM)%XP_QDAMAX=>XP_QDAMAX
PACK_GR_FIELD_MODEL(KFROM)%XP_T1AMAX=>XP_T1AMAX
PACK_GR_FIELD_MODEL(KFROM)%XP_T2AMAX=>XP_T2AMAX
PACK_GR_FIELD_MODEL(KFROM)%XP_WSAT=>XP_WSAT
PACK_GR_FIELD_MODEL(KFROM)%XP_WWILT=>XP_WWILT
PACK_GR_FIELD_MODEL(KFROM)%XP_WFC=>XP_WFC
PACK_GR_FIELD_MODEL(KFROM)%XP_BCOEF=>XP_BCOEF
PACK_GR_FIELD_MODEL(KFROM)%XP_CGSAT=>XP_CGSAT
PACK_GR_FIELD_MODEL(KFROM)%XP_C1SAT=>XP_C1SAT
PACK_GR_FIELD_MODEL(KFROM)%XP_C2REF=>XP_C2REF
PACK_GR_FIELD_MODEL(KFROM)%XP_C3=>XP_C3
PACK_GR_FIELD_MODEL(KFROM)%XP_ACOEF=>XP_ACOEF
PACK_GR_FIELD_MODEL(KFROM)%XP_PCOEF=>XP_PCOEF
PACK_GR_FIELD_MODEL(KFROM)%XP_HCAPSOIL=>XP_HCAPSOIL
PACK_GR_FIELD_MODEL(KFROM)%XP_CONDDRY=>XP_CONDDRY
PACK_GR_FIELD_MODEL(KFROM)%XP_CONDSLD=>XP_CONDSLD
PACK_GR_FIELD_MODEL(KFROM)%XP_TDEEP=>XP_TDEEP
PACK_GR_FIELD_MODEL(KFROM)%XP_GAMMAT=>XP_GAMMAT
PACK_GR_FIELD_MODEL(KFROM)%XP_TS_TOWN=>XP_TS_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_ALB_TOWN=>XP_ALB_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_T_CANYON=>XP_T_CANYON
PACK_GR_FIELD_MODEL(KFROM)%XP_R_CANYON=>XP_R_CANYON
PACK_GR_FIELD_MODEL(KFROM)%XP_Q_CANYON=>XP_Q_CANYON
PACK_GR_FIELD_MODEL(KFROM)%XP_U_CANYON=>XP_U_CANYON
PACK_GR_FIELD_MODEL(KFROM)%XP_TS_ROOF=>XP_TS_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_TS_ROAD=>XP_TS_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TS_WALL=>XP_TS_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_TI_ROAD=>XP_TI_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TI_BLD=>XP_TI_BLD
PACK_GR_FIELD_MODEL(KFROM)%XP_T_ROOF=>XP_T_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_T_ROAD=>XP_T_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_T_WALL=>XP_T_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_WS_ROOF=>XP_WS_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_WS_ROAD=>XP_WS_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_WSNOW_ROOF=>XP_WSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_TSNOW_ROOF=>XP_TSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_RSNOW_ROOF=>XP_RSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_ASNOW_ROOF=>XP_ASNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_TSSNOW_ROOF=>XP_TSSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_ESNOW_ROOF=>XP_ESNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_WSNOW_ROAD=>XP_WSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TSNOW_ROAD=>XP_TSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_RSNOW_ROAD=>XP_RSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ASNOW_ROAD=>XP_ASNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TSSNOW_ROAD=>XP_TSSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ESNOW_ROAD=>XP_ESNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0NAT=>XP_Z0NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0HNAT=>XP_Z0HNAT
PACK_GR_FIELD_MODEL(KFROM)%XP_ALB_ROAD=>XP_ALB_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ALB_ROOF=>XP_ALB_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_ALB_WALL=>XP_ALB_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_RN_ROOF=>XP_RN_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_H_ROOF=>XP_H_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_ROOF=>XP_LE_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_GFLUX_ROOF=>XP_GFLUX_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_RN_ROAD=>XP_RN_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_H_ROAD=>XP_H_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_ROAD=>XP_LE_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_GFLUX_ROAD=>XP_GFLUX_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_RN_WALL=>XP_RN_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_H_WALL=>XP_H_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_WALL=>XP_LE_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_GFLUX_WALL=>XP_GFLUX_WALL
PACK_GR_FIELD_MODEL(KFROM)%XP_RNSNOW_ROOF=>XP_RNSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_HSNOW_ROOF=>XP_HSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_LESNOW_ROOF=>XP_LESNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_GSNOW_ROOF=>XP_GSNOW_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_MELT_ROOF=>XP_MELT_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_RNSNOW_ROAD=>XP_RNSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_HSNOW_ROAD=>XP_HSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_LESNOW_ROAD=>XP_LESNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_GSNOW_ROAD=>XP_GSNOW_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_MELT_ROAD=>XP_MELT_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_RN_TOWN=>XP_RN_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_H_TOWN=>XP_H_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_TOWN=>XP_LE_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_GFLUX_TOWN=>XP_GFLUX_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_RUNOFF_TOWN=>XP_RUNOFF_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_SFU_TOWN=>XP_SFU_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_SFV_TOWN=>XP_SFV_TOWN
PACK_GR_FIELD_MODEL(KFROM)%XP_LES_ROOF=>XP_LES_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_LES_ROAD=>XP_LES_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_RUNOFF_ROOF=>XP_RUNOFF_ROOF
PACK_GR_FIELD_MODEL(KFROM)%XP_RUNOFF_ROAD=>XP_RUNOFF_ROAD
PACK_GR_FIELD_MODEL(KFROM)%XP_PS=>XP_PS
PACK_GR_FIELD_MODEL(KFROM)%XP_PA=>XP_PA
PACK_GR_FIELD_MODEL(KFROM)%XP_EXNS=>XP_EXNS
PACK_GR_FIELD_MODEL(KFROM)%XP_EXNA=>XP_EXNA
PACK_GR_FIELD_MODEL(KFROM)%XP_TA=>XP_TA
PACK_GR_FIELD_MODEL(KFROM)%XP_QA=>XP_QA
PACK_GR_FIELD_MODEL(KFROM)%XP_RHOA=>XP_RHOA
PACK_GR_FIELD_MODEL(KFROM)%XP_LW_RAD=>XP_LW_RAD
PACK_GR_FIELD_MODEL(KFROM)%XP_DIR_SW_RAD=>XP_DIR_SW_RAD
PACK_GR_FIELD_MODEL(KFROM)%XP_SCA_SW_RAD=>XP_SCA_SW_RAD
PACK_GR_FIELD_MODEL(KFROM)%XP_TANZEN=>XP_TANZEN
PACK_GR_FIELD_MODEL(KFROM)%XP_RAIN=>XP_RAIN
PACK_GR_FIELD_MODEL(KFROM)%XP_SNOW=>XP_SNOW
PACK_GR_FIELD_MODEL(KFROM)%XP_ZREF=>XP_ZREF
PACK_GR_FIELD_MODEL(KFROM)%XP_DIRCOSZW=>XP_DIRCOSZW
PACK_GR_FIELD_MODEL(KFROM)%XP_USLOPE=>XP_USLOPE
PACK_GR_FIELD_MODEL(KFROM)%XP_VSLOPE=>XP_VSLOPE
PACK_GR_FIELD_MODEL(KFROM)%XP_VMOD=>XP_VMOD
PACK_GR_FIELD_MODEL(KFROM)%XP_H_TRAFFIC=>XP_H_TRAFFIC
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_TRAFFIC=>XP_LE_TRAFFIC
PACK_GR_FIELD_MODEL(KFROM)%XP_H_INDUSTRY=>XP_H_INDUSTRY
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_INDUSTRY=>XP_LE_INDUSTRY
PACK_GR_FIELD_MODEL(KFROM)%LP_MOW=>LP_MOW
PACK_GR_FIELD_MODEL(KFROM)%XP_WG=>XP_WG
PACK_GR_FIELD_MODEL(KFROM)%XP_TG=>XP_TG
PACK_GR_FIELD_MODEL(KFROM)%XP_WGI=>XP_WGI
PACK_GR_FIELD_MODEL(KFROM)%XP_RS=>XP_RS
PACK_GR_FIELD_MODEL(KFROM)%XP_TS=>XP_TS
PACK_GR_FIELD_MODEL(KFROM)%XP_WR=>XP_WR
PACK_GR_FIELD_MODEL(KFROM)%XP_WSNOW_VEG=>XP_WSNOW_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_RSNOW_VEG=>XP_RSNOW_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_ASNOW_VEG=>XP_ASNOW_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_PSN=>XP_PSN
PACK_GR_FIELD_MODEL(KFROM)%XP_PSNV=>XP_PSNV
PACK_GR_FIELD_MODEL(KFROM)%XP_PSNG=>XP_PSNG
PACK_GR_FIELD_MODEL(KFROM)%XP_VEG=>XP_VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_LAI=>XP_LAI
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0VEG=>XP_Z0VEG
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0HVEG=>XP_Z0HVEG
PACK_GR_FIELD_MODEL(KFROM)%XP_ANFM=>XP_ANFM
PACK_GR_FIELD_MODEL(KFROM)%XP_ANF=>XP_ANF
PACK_GR_FIELD_MODEL(KFROM)%XP_WG1=>XP_WG1
PACK_GR_FIELD_MODEL(KFROM)%XP_T2=>XP_T2
PACK_GR_FIELD_MODEL(KFROM)%XP_WG2=>XP_WG2
PACK_GR_FIELD_MODEL(KFROM)%XP_WG3=>XP_WG3
PACK_GR_FIELD_MODEL(KFROM)%XP_WGI1=>XP_WGI1
PACK_GR_FIELD_MODEL(KFROM)%XP_WGI2=>XP_WGI2
PACK_GR_FIELD_MODEL(KFROM)%XP_SUNTIME=>XP_SUNTIME
PACK_GR_FIELD_MODEL(KFROM)%XP_SW_RAD=>XP_SW_RAD
PACK_GR_FIELD_MODEL(KFROM)%XP_ALFA=>XP_ALFA
PACK_GR_FIELD_MODEL(KFROM)%XP_ALB_ECO=>XP_ALB_ECO
PACK_GR_FIELD_MODEL(KFROM)%XP_THA=>XP_THA
PACK_GR_FIELD_MODEL(KFROM)%XP_D2=>XP_D2
PACK_GR_FIELD_MODEL(KFROM)%XP_D3=>XP_D3
PACK_GR_FIELD_MODEL(KFROM)%XP_WDRAIN=>XP_WDRAIN
PACK_GR_FIELD_MODEL(KFROM)%XP_RESA=>XP_RESA
PACK_GR_FIELD_MODEL(KFROM)%XP_CH_NAT=>XP_CH_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_CD_NAT=>XP_CD_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_CDN_NAT=>XP_CDN_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_RI_NAT=>XP_RI_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_HU_NAT=>XP_HU_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_HUG_NAT=>XP_HUG_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_RN_NAT=>XP_RN_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_H_NAT=>XP_H_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_NAT=>XP_LE_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LEG_NAT=>XP_LEG_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LEV_NAT=>XP_LEV_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LES_NAT=>XP_LES_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LER_NAT=>XP_LER_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_LETR_NAT=>XP_LETR_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_GFLUX_NAT=>XP_GFLUX_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_SFU_NAT=>XP_SFU_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_SFV_NAT=>XP_SFV_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_DRAIN_NAT=>XP_DRAIN_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_RUNOFF_NAT=>XP_RUNOFF_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_MELT_NAT=>XP_MELT_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_CO2FLUX_NAT=>XP_CO2FLUX_NAT
PACK_GR_FIELD_MODEL(KFROM)%XP_AN=>XP_AN
PACK_GR_FIELD_MODEL(KFROM)%XP_CG=>XP_CG
PACK_GR_FIELD_MODEL(KFROM)%XP_C1=>XP_C1
PACK_GR_FIELD_MODEL(KFROM)%XP_C2=>XP_C2
PACK_GR_FIELD_MODEL(KFROM)%XP_WGEQ=>XP_WGEQ
PACK_GR_FIELD_MODEL(KFROM)%XP_CT=>XP_CT
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0SEA=>XP_Z0SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_SST=>XP_SST
PACK_GR_FIELD_MODEL(KFROM)%XP_H_SEA=>XP_H_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_SEA=>XP_LE_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_SFU_SEA=>XP_SFU_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_SFV_SEA=>XP_SFV_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_CD_SEA=>XP_CD_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_CDN_SEA=>XP_CDN_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_CH_SEA=>XP_CH_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_RI_SEA=>XP_RI_SEA
PACK_GR_FIELD_MODEL(KFROM)%XP_Z0WATER=>XP_Z0WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_TS_WATER=>XP_TS_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_H_WATER=>XP_H_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_LE_WATER=>XP_LE_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_SFU_WATER=>XP_SFU_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_SFV_WATER=>XP_SFV_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_CD_WATER=>XP_CD_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_CDN_WATER=>XP_CDN_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_CH_WATER=>XP_CH_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_RI_WATER=>XP_RI_WATER
PACK_GR_FIELD_MODEL(KFROM)%XP_CH_TOWN=>XP_CH_TOWN
!
! Current model is set to model KTO
NSIZE_NATURE=>PACK_GR_FIELD_MODEL(KTO)%NSIZE_NATURE
NSIZE_TOWN=>PACK_GR_FIELD_MODEL(KTO)%NSIZE_TOWN
NSIZE_SEA=>PACK_GR_FIELD_MODEL(KTO)%NSIZE_SEA
NSIZE_WATER=>PACK_GR_FIELD_MODEL(KTO)%NSIZE_WATER
N1_NATURE=>PACK_GR_FIELD_MODEL(KTO)%N1_NATURE
N2_NATURE=>PACK_GR_FIELD_MODEL(KTO)%N2_NATURE
N1_TOWN=>PACK_GR_FIELD_MODEL(KTO)%N1_TOWN
N2_TOWN=>PACK_GR_FIELD_MODEL(KTO)%N2_TOWN
N1_SEA=>PACK_GR_FIELD_MODEL(KTO)%N1_SEA
N2_SEA=>PACK_GR_FIELD_MODEL(KTO)%N2_SEA
N1_WATER=>PACK_GR_FIELD_MODEL(KTO)%N1_WATER
N2_WATER=>PACK_GR_FIELD_MODEL(KTO)%N2_WATER
XP_AOSIP=>PACK_GR_FIELD_MODEL(KTO)%XP_AOSIP
XP_AOSIM=>PACK_GR_FIELD_MODEL(KTO)%XP_AOSIM
XP_AOSJP=>PACK_GR_FIELD_MODEL(KTO)%XP_AOSJP
XP_AOSJM=>PACK_GR_FIELD_MODEL(KTO)%XP_AOSJM
XP_HO2IP=>PACK_GR_FIELD_MODEL(KTO)%XP_HO2IP
XP_HO2IM=>PACK_GR_FIELD_MODEL(KTO)%XP_HO2IM
XP_HO2JP=>PACK_GR_FIELD_MODEL(KTO)%XP_HO2JP
XP_HO2JM=>PACK_GR_FIELD_MODEL(KTO)%XP_HO2JM
XP_Z0REL=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0REL
XP_Z0EFFIM=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0EFFIM
XP_Z0EFFIP=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0EFFIP
XP_Z0EFFJM=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0EFFJM
XP_Z0EFFJP=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0EFFJP
XP_Z0EFF=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0EFF
XP_SSO_STDEV=>PACK_GR_FIELD_MODEL(KTO)%XP_SSO_STDEV
XP_SSO_ANISOTROPY=>PACK_GR_FIELD_MODEL(KTO)%XP_SSO_ANISOTROPY
XP_SSO_DIRECTION=>PACK_GR_FIELD_MODEL(KTO)%XP_SSO_DIRECTION
XP_SSO_SLOPE=>PACK_GR_FIELD_MODEL(KTO)%XP_SSO_SLOPE
XP_Z0_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0_TOWN
XP_ALBNIR_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBNIR_TOWN
XP_ALBVIS_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBVIS_TOWN
XP_EMIS_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_EMIS_TOWN
XP_ALBNIR_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBNIR_ROOF
XP_ALBVIS_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBVIS_ROOF
XP_EMIS_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_EMIS_ROOF
XP_HC_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_HC_ROOF
XP_TC_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_TC_ROOF
XP_D_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_D_ROOF
XP_ALBNIR_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBNIR_ROAD
XP_ALBVIS_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBVIS_ROAD
XP_EMIS_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_EMIS_ROAD
XP_HC_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_HC_ROAD
XP_TC_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_TC_ROAD
XP_D_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_D_ROAD
XP_SVF_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_SVF_ROAD
XP_ALBNIR_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBNIR_WALL
XP_ALBVIS_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBVIS_WALL
XP_EMIS_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_EMIS_WALL
XP_HC_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_HC_WALL
XP_TC_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_TC_WALL
XP_D_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_D_WALL
XP_SVF_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_SVF_WALL
XP_BLD_HEIGHT=>PACK_GR_FIELD_MODEL(KTO)%XP_BLD_HEIGHT
XP_BLD_HL_RATIO=>PACK_GR_FIELD_MODEL(KTO)%XP_BLD_HL_RATIO
XP_BLD=>PACK_GR_FIELD_MODEL(KTO)%XP_BLD
XP_CAN_HW_RATIO=>PACK_GR_FIELD_MODEL(KTO)%XP_CAN_HW_RATIO
XP_CLAY=>PACK_GR_FIELD_MODEL(KTO)%XP_CLAY
XP_SAND=>PACK_GR_FIELD_MODEL(KTO)%XP_SAND
XP_RSMIN=>PACK_GR_FIELD_MODEL(KTO)%XP_RSMIN
XP_GAMMA=>PACK_GR_FIELD_MODEL(KTO)%XP_GAMMA
XP_RGL=>PACK_GR_FIELD_MODEL(KTO)%XP_RGL
XP_CV=>PACK_GR_FIELD_MODEL(KTO)%XP_CV
XP_ALBNIR_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBNIR_VEG
XP_ALBVIS_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_ALBVIS_VEG
XP_EMIS_ECO=>PACK_GR_FIELD_MODEL(KTO)%XP_EMIS_ECO
XP_D_G=>PACK_GR_FIELD_MODEL(KTO)%XP_D_G
XP_C4B=>PACK_GR_FIELD_MODEL(KTO)%XP_C4B
XP_C4REF=>PACK_GR_FIELD_MODEL(KTO)%XP_C4REF
XP_RUNOFFB=>PACK_GR_FIELD_MODEL(KTO)%XP_RUNOFFB
XP_TAUICE=>PACK_GR_FIELD_MODEL(KTO)%XP_TAUICE
XP_CO2FLUX=>PACK_GR_FIELD_MODEL(KTO)%XP_CO2FLUX
XP_VEGTYPE=>PACK_GR_FIELD_MODEL(KTO)%XP_VEGTYPE
XP_BSLAI=>PACK_GR_FIELD_MODEL(KTO)%XP_BSLAI
XP_LAIMIN=>PACK_GR_FIELD_MODEL(KTO)%XP_LAIMIN
XP_SEFOLD=>PACK_GR_FIELD_MODEL(KTO)%XP_SEFOLD
XP_H_TREE=>PACK_GR_FIELD_MODEL(KTO)%XP_H_TREE
XP_CSP=>PACK_GR_FIELD_MODEL(KTO)%XP_CSP
XP_ANMAX=>PACK_GR_FIELD_MODEL(KTO)%XP_ANMAX
XP_GMES=>PACK_GR_FIELD_MODEL(KTO)%XP_GMES
XP_FZERO=>PACK_GR_FIELD_MODEL(KTO)%XP_FZERO
XP_EPSO=>PACK_GR_FIELD_MODEL(KTO)%XP_EPSO
XP_GAMM=>PACK_GR_FIELD_MODEL(KTO)%XP_GAMM
XP_QDGAMM=>PACK_GR_FIELD_MODEL(KTO)%XP_QDGAMM
XP_QDGMES=>PACK_GR_FIELD_MODEL(KTO)%XP_QDGMES
XP_T1GMES=>PACK_GR_FIELD_MODEL(KTO)%XP_T1GMES
XP_T2GMES=>PACK_GR_FIELD_MODEL(KTO)%XP_T2GMES
XP_AMAX=>PACK_GR_FIELD_MODEL(KTO)%XP_AMAX
XP_QDAMAX=>PACK_GR_FIELD_MODEL(KTO)%XP_QDAMAX
XP_T1AMAX=>PACK_GR_FIELD_MODEL(KTO)%XP_T1AMAX
XP_T2AMAX=>PACK_GR_FIELD_MODEL(KTO)%XP_T2AMAX
XP_WSAT=>PACK_GR_FIELD_MODEL(KTO)%XP_WSAT
XP_WWILT=>PACK_GR_FIELD_MODEL(KTO)%XP_WWILT
XP_WFC=>PACK_GR_FIELD_MODEL(KTO)%XP_WFC
XP_BCOEF=>PACK_GR_FIELD_MODEL(KTO)%XP_BCOEF
XP_CGSAT=>PACK_GR_FIELD_MODEL(KTO)%XP_CGSAT
XP_C1SAT=>PACK_GR_FIELD_MODEL(KTO)%XP_C1SAT
XP_C2REF=>PACK_GR_FIELD_MODEL(KTO)%XP_C2REF
XP_C3=>PACK_GR_FIELD_MODEL(KTO)%XP_C3
XP_ACOEF=>PACK_GR_FIELD_MODEL(KTO)%XP_ACOEF
XP_PCOEF=>PACK_GR_FIELD_MODEL(KTO)%XP_PCOEF
XP_HCAPSOIL=>PACK_GR_FIELD_MODEL(KTO)%XP_HCAPSOIL
XP_CONDDRY=>PACK_GR_FIELD_MODEL(KTO)%XP_CONDDRY
XP_CONDSLD=>PACK_GR_FIELD_MODEL(KTO)%XP_CONDSLD
XP_TDEEP=>PACK_GR_FIELD_MODEL(KTO)%XP_TDEEP
XP_GAMMAT=>PACK_GR_FIELD_MODEL(KTO)%XP_GAMMAT
XP_TS_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_TS_TOWN
XP_ALB_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_ALB_TOWN
XP_T_CANYON=>PACK_GR_FIELD_MODEL(KTO)%XP_T_CANYON
XP_R_CANYON=>PACK_GR_FIELD_MODEL(KTO)%XP_R_CANYON
XP_Q_CANYON=>PACK_GR_FIELD_MODEL(KTO)%XP_Q_CANYON
XP_U_CANYON=>PACK_GR_FIELD_MODEL(KTO)%XP_U_CANYON
XP_TS_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_TS_ROOF
XP_TS_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_TS_ROAD
XP_TS_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_TS_WALL
XP_TI_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_TI_ROAD
XP_TI_BLD=>PACK_GR_FIELD_MODEL(KTO)%XP_TI_BLD
XP_T_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_T_ROOF
XP_T_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_T_ROAD
XP_T_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_T_WALL
XP_WS_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_WS_ROOF
XP_WS_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_WS_ROAD
XP_WSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_WSNOW_ROOF
XP_TSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_TSNOW_ROOF
XP_RSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_RSNOW_ROOF
XP_ASNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_ASNOW_ROOF
XP_TSSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_TSSNOW_ROOF
XP_ESNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_ESNOW_ROOF
XP_WSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_WSNOW_ROAD
XP_TSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_TSNOW_ROAD
XP_RSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_RSNOW_ROAD
XP_ASNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_ASNOW_ROAD
XP_TSSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_TSSNOW_ROAD
XP_ESNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_ESNOW_ROAD
XP_Z0NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0NAT
XP_Z0HNAT=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0HNAT
XP_ALB_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_ALB_ROAD
XP_ALB_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_ALB_ROOF
XP_ALB_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_ALB_WALL
XP_RN_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_RN_ROOF
XP_H_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_H_ROOF
XP_LE_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_ROOF
XP_GFLUX_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_GFLUX_ROOF
XP_RN_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_RN_ROAD
XP_H_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_H_ROAD
XP_LE_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_ROAD
XP_GFLUX_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_GFLUX_ROAD
XP_RN_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_RN_WALL
XP_H_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_H_WALL
XP_LE_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_WALL
XP_GFLUX_WALL=>PACK_GR_FIELD_MODEL(KTO)%XP_GFLUX_WALL
XP_RNSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_RNSNOW_ROOF
XP_HSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_HSNOW_ROOF
XP_LESNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_LESNOW_ROOF
XP_GSNOW_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_GSNOW_ROOF
XP_MELT_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_MELT_ROOF
XP_RNSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_RNSNOW_ROAD
XP_HSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_HSNOW_ROAD
XP_LESNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_LESNOW_ROAD
XP_GSNOW_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_GSNOW_ROAD
XP_MELT_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_MELT_ROAD
XP_RN_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_RN_TOWN
XP_H_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_H_TOWN
XP_LE_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_TOWN
XP_GFLUX_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_GFLUX_TOWN
XP_RUNOFF_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_RUNOFF_TOWN
XP_SFU_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_SFU_TOWN
XP_SFV_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_SFV_TOWN
XP_LES_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_LES_ROOF
XP_LES_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_LES_ROAD
XP_RUNOFF_ROOF=>PACK_GR_FIELD_MODEL(KTO)%XP_RUNOFF_ROOF
XP_RUNOFF_ROAD=>PACK_GR_FIELD_MODEL(KTO)%XP_RUNOFF_ROAD
XP_PS=>PACK_GR_FIELD_MODEL(KTO)%XP_PS
XP_PA=>PACK_GR_FIELD_MODEL(KTO)%XP_PA
XP_EXNS=>PACK_GR_FIELD_MODEL(KTO)%XP_EXNS
XP_EXNA=>PACK_GR_FIELD_MODEL(KTO)%XP_EXNA
XP_TA=>PACK_GR_FIELD_MODEL(KTO)%XP_TA
XP_QA=>PACK_GR_FIELD_MODEL(KTO)%XP_QA
XP_RHOA=>PACK_GR_FIELD_MODEL(KTO)%XP_RHOA
XP_LW_RAD=>PACK_GR_FIELD_MODEL(KTO)%XP_LW_RAD
XP_DIR_SW_RAD=>PACK_GR_FIELD_MODEL(KTO)%XP_DIR_SW_RAD
XP_SCA_SW_RAD=>PACK_GR_FIELD_MODEL(KTO)%XP_SCA_SW_RAD
XP_TANZEN=>PACK_GR_FIELD_MODEL(KTO)%XP_TANZEN
XP_RAIN=>PACK_GR_FIELD_MODEL(KTO)%XP_RAIN
XP_SNOW=>PACK_GR_FIELD_MODEL(KTO)%XP_SNOW
XP_ZREF=>PACK_GR_FIELD_MODEL(KTO)%XP_ZREF
XP_DIRCOSZW=>PACK_GR_FIELD_MODEL(KTO)%XP_DIRCOSZW
XP_USLOPE=>PACK_GR_FIELD_MODEL(KTO)%XP_USLOPE
XP_VSLOPE=>PACK_GR_FIELD_MODEL(KTO)%XP_VSLOPE
XP_VMOD=>PACK_GR_FIELD_MODEL(KTO)%XP_VMOD
XP_H_TRAFFIC=>PACK_GR_FIELD_MODEL(KTO)%XP_H_TRAFFIC
XP_LE_TRAFFIC=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_TRAFFIC
XP_H_INDUSTRY=>PACK_GR_FIELD_MODEL(KTO)%XP_H_INDUSTRY
XP_LE_INDUSTRY=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_INDUSTRY
LP_MOW=>PACK_GR_FIELD_MODEL(KTO)%LP_MOW
XP_WG=>PACK_GR_FIELD_MODEL(KTO)%XP_WG
XP_TG=>PACK_GR_FIELD_MODEL(KTO)%XP_TG
XP_WGI=>PACK_GR_FIELD_MODEL(KTO)%XP_WGI
XP_RS=>PACK_GR_FIELD_MODEL(KTO)%XP_RS
XP_TS=>PACK_GR_FIELD_MODEL(KTO)%XP_TS
XP_WR=>PACK_GR_FIELD_MODEL(KTO)%XP_WR
XP_WSNOW_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_WSNOW_VEG
XP_RSNOW_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_RSNOW_VEG
XP_ASNOW_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_ASNOW_VEG
XP_PSN=>PACK_GR_FIELD_MODEL(KTO)%XP_PSN
XP_PSNV=>PACK_GR_FIELD_MODEL(KTO)%XP_PSNV
XP_PSNG=>PACK_GR_FIELD_MODEL(KTO)%XP_PSNG
XP_VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_VEG
XP_LAI=>PACK_GR_FIELD_MODEL(KTO)%XP_LAI
XP_Z0VEG=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0VEG
XP_Z0HVEG=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0HVEG
XP_ANFM=>PACK_GR_FIELD_MODEL(KTO)%XP_ANFM
XP_ANF=>PACK_GR_FIELD_MODEL(KTO)%XP_ANF
XP_WG1=>PACK_GR_FIELD_MODEL(KTO)%XP_WG1
XP_T2=>PACK_GR_FIELD_MODEL(KTO)%XP_T2
XP_WG2=>PACK_GR_FIELD_MODEL(KTO)%XP_WG2
XP_WG3=>PACK_GR_FIELD_MODEL(KTO)%XP_WG3
XP_WGI1=>PACK_GR_FIELD_MODEL(KTO)%XP_WGI1
XP_WGI2=>PACK_GR_FIELD_MODEL(KTO)%XP_WGI2
XP_SUNTIME=>PACK_GR_FIELD_MODEL(KTO)%XP_SUNTIME
XP_SW_RAD=>PACK_GR_FIELD_MODEL(KTO)%XP_SW_RAD
XP_ALFA=>PACK_GR_FIELD_MODEL(KTO)%XP_ALFA
XP_ALB_ECO=>PACK_GR_FIELD_MODEL(KTO)%XP_ALB_ECO
XP_THA=>PACK_GR_FIELD_MODEL(KTO)%XP_THA
XP_D2=>PACK_GR_FIELD_MODEL(KTO)%XP_D2
XP_D3=>PACK_GR_FIELD_MODEL(KTO)%XP_D3
XP_WDRAIN=>PACK_GR_FIELD_MODEL(KTO)%XP_WDRAIN
XP_RESA=>PACK_GR_FIELD_MODEL(KTO)%XP_RESA
XP_CH_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_CH_NAT
XP_CD_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_CD_NAT
XP_CDN_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_CDN_NAT
XP_RI_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_RI_NAT
XP_HU_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_HU_NAT
XP_HUG_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_HUG_NAT
XP_RN_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_RN_NAT
XP_H_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_H_NAT
XP_LE_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_NAT
XP_LEG_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LEG_NAT
XP_LEV_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LEV_NAT
XP_LES_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LES_NAT
XP_LER_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LER_NAT
XP_LETR_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_LETR_NAT
XP_GFLUX_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_GFLUX_NAT
XP_SFU_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_SFU_NAT
XP_SFV_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_SFV_NAT
XP_DRAIN_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_DRAIN_NAT
XP_RUNOFF_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_RUNOFF_NAT
XP_MELT_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_MELT_NAT
XP_CO2FLUX_NAT=>PACK_GR_FIELD_MODEL(KTO)%XP_CO2FLUX_NAT
XP_AN=>PACK_GR_FIELD_MODEL(KTO)%XP_AN
XP_CG=>PACK_GR_FIELD_MODEL(KTO)%XP_CG
XP_C1=>PACK_GR_FIELD_MODEL(KTO)%XP_C1
XP_C2=>PACK_GR_FIELD_MODEL(KTO)%XP_C2
XP_WGEQ=>PACK_GR_FIELD_MODEL(KTO)%XP_WGEQ
XP_CT=>PACK_GR_FIELD_MODEL(KTO)%XP_CT
XP_Z0SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0SEA
XP_SST=>PACK_GR_FIELD_MODEL(KTO)%XP_SST
XP_H_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_H_SEA
XP_LE_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_SEA
XP_SFU_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_SFU_SEA
XP_SFV_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_SFV_SEA
XP_CD_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_CD_SEA
XP_CDN_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_CDN_SEA
XP_CH_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_CH_SEA
XP_RI_SEA=>PACK_GR_FIELD_MODEL(KTO)%XP_RI_SEA
XP_Z0WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_Z0WATER
XP_TS_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_TS_WATER
XP_H_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_H_WATER
XP_LE_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_LE_WATER
XP_SFU_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_SFU_WATER
XP_SFV_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_SFV_WATER
XP_CD_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_CD_WATER
XP_CDN_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_CDN_WATER
XP_CH_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_CH_WATER
XP_RI_WATER=>PACK_GR_FIELD_MODEL(KTO)%XP_RI_WATER
XP_CH_TOWN=>PACK_GR_FIELD_MODEL(KTO)%XP_CH_TOWN

END SUBROUTINE PACK_GR_FIELD_GOTO_MODEL

END MODULE MODD_PACK_GR_FIELD_n
